/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packeteditor;

import java.awt.*;
import java.util.*;

public class AbsoluteLayout implements LayoutManager2 {

    /* private variables */
    private Hashtable components;

    /* Constructor */
    public AbsoluteLayout() {
	components = new Hashtable();
    }

    /* LayoutManager methods */

    /**
     * This method is not supported by AbsoluteLayout, 
     * use addLayoutComponent(Component comp, Object constraints)
     * instead.
     */
    public void addLayoutComponent(String name, Component comp) {
	throw new IllegalArgumentException("Operation not supported");
    }
    
    /**
     * Removes component from this Layout.
     * 
     * @param comp The component to remove
     */
    public void removeLayoutComponent(Component comp) {
	components.remove(comp);
    }

    /**
     * Calculate suitable dimensions for a panel that 
     * contains the components of this layout.
     *
     * @param parent The parent component that is laid out
     * @return The dimensions
     */
    public Dimension preferredLayoutSize(Container parent) {
	int totalWidth = 0, totalHeight = 0;

	Enumeration e = components.keys();
	while(e.hasMoreElements()) {
	    Component component = (Component) e.nextElement();
	    AbsoluteConstraints constraint = (AbsoluteConstraints) components.get(component);
	    Dimension dim = component.getPreferredSize();

	    int w = constraint.getWidth();
	    if(w < 0)
		w = dim.width;

	    int h = constraint.getHeight();
	    if(h < 0)
		h = dim.height;

	    int x = constraint.getX(); 
	    if((x + w) > totalWidth)
		totalWidth = x + w;

	    int y = constraint.getY(); 
	    if((y + h) > totalHeight)
		totalHeight = y + h;
	}

	return(new Dimension(totalWidth, totalHeight));
    }

    public Dimension minimumLayoutSize(Container parent) {
	int totalWidth = 0, totalHeight = 0;

	Enumeration e = components.keys();
	while(e.hasMoreElements()) {
	    Component component = (Component) e.nextElement();
	    AbsoluteConstraints constraint = (AbsoluteConstraints) components.get(component);
	    Dimension dim = component.getMinimumSize();

	    int w = constraint.getWidth();
	    if(w < 0)
		w = dim.width;

	    int h = constraint.getHeight();
	    if(h < 0)
		h = dim.height;

	    int x = constraint.getX(); 
	    if((x + w) > totalWidth)
		totalWidth = x + w;

	    int y = constraint.getY(); 
	    if((y + h) > totalHeight)
		totalHeight = y + h;
	}

	return(new Dimension(totalWidth, totalHeight));
    }

    public void layoutContainer(Container parent) {
	Enumeration e = components.keys();
	while(e.hasMoreElements()) {
	    Component component = (Component) e.nextElement();
	    AbsoluteConstraints constraint = (AbsoluteConstraints) components.get(component);
	    Dimension dim = component.getPreferredSize();

	    int w = constraint.getWidth();
	    if(w < 0)
		w = dim.width;

	    int h = constraint.getHeight();
	    if(h < 0)
		h = dim.height;

	    component.setBounds(constraint.getX(), constraint.getY(), w, h);
	}
    }

    /* LayoutManager2 methods */

    public void addLayoutComponent(Component comp, Object constraints) {
	if(!(constraints instanceof AbsoluteConstraints))
	    throw new IllegalArgumentException("Invalid constraints Object");
	components.put(comp, constraints);
    }

    public Dimension maximumLayoutSize(Container target) {
        return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }

    public float getLayoutAlignmentX(Container target) {
	return(0);
    }

    public float getLayoutAlignmentY(Container target) {
	return(0);
    }

    public void invalidateLayout(Container target) {
	/* Do nothing */
    }
}
