/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packeteditor;

import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import org.w3c.dom.*;

import packetengine.*;

/**
 * class <code> Action.java </code>
 * implements funtionalities of the 
 * user interface
 *
 * @author Kimmo Ahola, Titta Koskinen
 * @version 1.0
 *
 * Created on 19. keskuuta 2002, 16:26
 */

public class Action {

    private PacketEngine packetEngine = null;
    private IPPacketEditor iPPacketEditor = null;
    private int iPacketNumber = 1;
    private int iProtocolNumber = 1;
    private DefaultMutableTreeNode node;
    private Document document = null;
    private StringBuffer openedFilePath = null;
    private WindowMessageOutput wmo = null;
    private int iDebugLevel = 1;
    /**
     *  <code>errorMessage</code> instance of errorMessage.
     */
    public ErrorMessage errorMessage = null;

    /**
     *  <code>parameters</code> parameters for network interface
     */
    public String parameters = "";



    /**
     * Creates a new <code>Action</code> instance.
     *
     * @param _iPPacketEditor an <code>IPPacketEditor</code> value
     */
    public Action(IPPacketEditor _iPPacketEditor) {
	iPPacketEditor = _iPPacketEditor;
	errorMessage = new ErrorMessage (iPPacketEditor);
    }

    /**
     * <code>getProtocolEntities</code> method
     * brings models of protocol entities from
     * Packet Engine to the Packet Editor
     *
     * @param address is a <code>String</code> address to the Packet Engine 
     * @return a <code>Vector</code> of protocol elements
     */
    public Vector getProtocolEntities(String address) {        

	Vector protocols;
	protocols = new Vector();
        Vector protocolDescriptors = null;
	protocolDescriptors = PacketEngine.getProtocolTags();
	
	if (protocolDescriptors != null){

	    for (Enumeration e = protocolDescriptors.elements(); e.hasMoreElements();) {
		Object o = e.nextElement();
		if(o instanceof String)
		    protocols.addElement(o);
		else {
		    ProtocolDescriptor protoDesc = (ProtocolDescriptor) o;
		    Protocol proto = new Protocol(protoDesc.tag, protoDesc.name);
		    //LinkedHashMap to keep the order of params
		    for (Iterator i = protoDesc.tags.keySet().iterator(); i.hasNext();) {
			String tag = (String) i.next();
			String name = (String) protoDesc.tags.get((Object)tag);
			proto.setParameter(tag, name, "");
		    }
		    protocols.addElement((Object)proto);
		}
	    }
	}

	return protocols;
    }

    /**
     * <code>newPacketSet</code> creates new packetset.
     *
     */
    public void newPacketSet ()
    {

	//show Save Packet Set Dialog
	JOptionPane savepane = new JOptionPane ("Save PacketSet?");
	Object [] options = new String []{"Save", "Reject", "Cancel"};
	savepane.setOptions(options);
	JDialog dialog = savepane.createDialog(iPPacketEditor.jMain, "Save PacketSet?");
	dialog.show();
	Object obj = savepane.getValue();
	if (obj.equals ("Save")){
	    savePacketSet();
	    openNewTree();
	    
	}
	else if(obj.equals ("Reject")){
	    openNewTree();
	}
    }

    /**
     * <code>openPacketSet</code> opens 
     * saved packetset.
     *
     */
    public void openPacketSet ()
    {        
	XMLDocument xmlDocument = null;

	iPacketNumber = 1;
	File dir = new File (".");
        JFileChooser fileChooser = new JFileChooser();
	PEFileFilter filter = new PEFileFilter("txt","Text file" );
	fileChooser.addChoosableFileFilter(filter);
        fileChooser.setCurrentDirectory(dir);
        if (fileChooser.showOpenDialog(iPPacketEditor.jMain) == JFileChooser.APPROVE_OPTION){
	    openedFilePath = new StringBuffer(fileChooser.getSelectedFile().getPath());
	    
	    try {
		FileInputStream fileInputStream = new FileInputStream(openedFilePath.toString());
		xmlDocument = new XMLDocument();
		document = xmlDocument.createDOMDocument(fileInputStream);
	    }
	    catch (FileNotFoundException exp) {
		errorMessage.printErrorMessage("File " + openedFilePath.toString() + " could not be found!");		
	    }
        }

	// Start parsing the DOM document to the Tree...
	documentToTree();
    }

    /**
     * <code>savePacketSet</code> saves created packetset.
     *
     */
    public void savePacketSet ()
    {
	if (openedFilePath == null){
	    savePacketSetAs();
	}
	else{
	    String XMLPacket = xmlToString();
	    try {
		FileWriter fileWriter = new FileWriter(openedFilePath.toString());
		fileWriter.write(XMLPacket);
		fileWriter.close();
	    }
	    catch(Exception e) {
		errorMessage.printErrorMessage("Saving failed\n" + e);
	    }	    	    
	} 
    }

    /**
     * <code>savePacketSetAs</code> saves created packetset with a new name.
     *
     */
    public void savePacketSetAs ()
    {
	String XMLPacket = xmlToString();
	if (XMLPacket != null){
	    saveStringAs(XMLPacket);
	}
        else
	   errorMessage.printErrorMessage("Creation of the DOM document failed");
    }

    /**
     * <code>addToPacket</code> adds selected protocol entity
     * from tabbedpane to the packet.
     *
     */
    public void addToPacket ()
    {
	int masterIndex = iPPacketEditor.getTabbedPaneMaster().getSelectedIndex();
	JTabbedPane[] slaves = iPPacketEditor.getTabbedPaneSlave();
        JTable selected = (JTable) slaves[masterIndex].getSelectedComponent();
        if (selected != null){
            ParameterTableModel model = (ParameterTableModel) selected.getModel();
            Protocol proto = model.getProtocol();
            Protocol cloned = (Protocol) proto.clone();
            node = iPPacketEditor.getSelectedTreeNode();

            if(node != null){
                // If selected is not packet then jTreeProto should be null
                Protocol jTreeProto = ((OidNode)node.getUserObject()).getProtocol();
                        
                if ((node.getParent() != null) && ( jTreeProto == null) ){
                    addToTree(cloned);
                }
            }
        }
    }

    /**
     * <code>removeFromPacket</code> removes selected protocol entity
     * from packet.
     *
     */
    public void removeFromPacket ()
    {
        node = iPPacketEditor.getSelectedTreeNode();
        if (node != null){
            MutableTreeNode parent = (MutableTreeNode) node.getParent();
            if (parent != null)
                parent.remove((MutableTreeNode)iPPacketEditor.getSelectedTreeNode());
        }
    }

    /**
     *  <code>setSendTime</code> ables user to modify packet's 
     *  sending time
     *
     */
    public void setSendTime ()
    {   
	node = iPPacketEditor.getSelectedTreeNode();
	if (node != null){
	    JOptionPane timepane = new JOptionPane ();
	    String inputValue = timepane.showInputDialog(iPPacketEditor.jMain,"Enter sending time to packet (ms)",
						     "Edit Packet",JOptionPane.QUESTION_MESSAGE);
	    if (inputValue != null){
		if (inputValue.equals("")==false && inputValue.equals("0") == false){
		    try {
			Integer iInputValue = new Integer (inputValue);
			if (iInputValue.intValue() >= 1 && iInputValue.intValue() <=  30000 ){
			    OidNode oidNode = (OidNode) node.getUserObject();	    
			    oidNode.setSendTime(inputValue);
			}
		    }
		    catch (NumberFormatException nfe){
			errorMessage.printErrorMessage ("Integer value expected");
		    }
		}
	    }
	}
	
    }

    /**
     *  <code>openDebugWindow</code> opens debug window.
     *  If WindowMessageOutput does not exist, creates it.
     *
     */
    public void openDebugWindow()
    {
	if(wmo == null) {
	    wmo = new WindowMessageOutput();
	    wmo.init();
	    wmo.openWindow(); 
	}
	else
	    wmo.openWindow(); 
    }

    /**
     *  <code>closeDebugWindow</code> opens debug window.
     *  If WindowMessageOutput does not exist, creates it.
     *
     */
    public void closeDebugWindow()
    {
	if(wmo != null) 
	    wmo.closeWindow(); 
    }

    /**
     * <code>setDebugLevel</code> ables user to modify program's
     *  debug level
     *  Level of debug output: 0 = none, 5 = everything, 1 = default
     *
     * @param level an <code>int</code> value
     */
    public void setDebugLevel(int level)
    {
	//System.out.println("debug level: " + level);
	iDebugLevel = level;
	PacketEngine.setDebugLevel(level);
    }


    /**
     * <code> setNetworkIf </code> opens dialog
     * that includes possible network interfaces
     * and text area for needed parameters.
     *
     * Includes inner class that implements
     * Action Listener for opened dialog. 
     * 
    */
    public void setNetworkIf ()
    {
  	//JButton okButton;
	final JButton cancelButton = new JButton ("Cancel");
	final JButton okButton = new JButton ("OK");
	final JButton clearButton = new JButton ("Clear");
	ButtonGroup buttonGroup = new ButtonGroup();
	
	//Parameters label and text area
	JLabel lParameters = new JLabel (" Parameters");
	final JTextArea tParameters = new JTextArea(1, 30);
	tParameters.insert (parameters,0);
	tParameters.setLineWrap(true);

	//Frame and panels
	final JFrame frame = new JFrame("Select network interface");
	JPanel panel = new JPanel(new BorderLayout());
	JPanel panel2 = new JPanel(new GridLayout (2,1));
	JPanel panelrButtons = new JPanel();
	JPanel panelParameters = new JPanel(new GridLayout (2,1));
	JPanel panelButtons = new JPanel();
	panelParameters.add (lParameters);

	//Get interfaces from Packet Engine 
	packetEngine = getPacketEngine();
	final String [] interfaces = PacketEngine.getNetworkInterfaces();
	final JRadioButton [] buttonTable = new JRadioButton[interfaces.length];

	String currentIF = packetEngine.getNetworkInterface();
	for (int i=0; i<interfaces.length; i++){
	    buttonTable [i] = new JRadioButton (interfaces[i], interfaces[i].equals(currentIF));
	    buttonGroup.add (buttonTable[i]);
	    panelrButtons.add (buttonTable[i]);	    
	}

	ActionListener listener = new ActionListener(){
		public void actionPerformed (ActionEvent evt){
		    Object source = (Object) evt.getSource();
		    if (source == okButton){
			for (int i = 0; i < buttonTable.length; i++){
			    if (buttonTable[i].isSelected()){
				parameters = tParameters.getText();
				setSelectedNetworkIf(buttonTable[i].getText(), tParameters.getText());
 				frame.setVisible(false);
 				break;
			    }
			}
		    }
		    else if (source == clearButton)
			tParameters.setText("");

		    else if (source == cancelButton)
			frame.setVisible(false);
		}
	    };

	//Add Actionlisteners to buttons
	okButton.addActionListener(listener);
	cancelButton.addActionListener(listener);
	clearButton.addActionListener(listener);

	//Layout
	panelParameters.add(tParameters);
	panelButtons.add (clearButton);
	panelButtons.add (okButton);
	panelButtons.add (cancelButton);
	panel2.add (panelrButtons);
	panel2.add (panelParameters);
	panel.add (panel2, BorderLayout.CENTER);
	panel.add (panelButtons, BorderLayout.SOUTH);
	frame.getContentPane().add (panel);
	frame.setSize(400,300);
 	Toolkit t = Toolkit.getDefaultToolkit();
 	Dimension d = t.getScreenSize();
 	int screenHeight = d.height;
 	int screenWidth = d.width;
 	frame.setLocation (screenWidth / 3, screenHeight / 2);
	frame.show();

     }

    /**
     * <code> setSelectedNetworkIf </code>
     * Sets selected interface and given parameters for the network interface.
     *
     * @param selected is a <code>String</code> presenting selected network interface tag. 
     * @param params <code>String</code> parameters for selected network interface. 
     */
    public void setSelectedNetworkIf(String selected, String params){
	boolean bOk = packetEngine.setNetworkInterface(selected, params);
	if (!bOk)	    
	    errorMessage.printErrorMessage ("Setting network interface or its parameters failed");
    }

    /**
     * <code>newPacket</code> creates new packet.
     *
     */
    public void newPacket ()
    {
        DefaultTreeModel model = null;
	//Open dialog for sending time
	String [] options = {"OK","Cancel"};
	JOptionPane timepane = new JOptionPane ();
	String inputValue = timepane.showInputDialog(iPPacketEditor.jMain,"Enter sending time to packet (ms)",
							"New Packet",JOptionPane.QUESTION_MESSAGE);
	if (inputValue != null){
	    if (inputValue.equals("")==false && inputValue.equals("0") == false){
		try {
		    Integer iInputValue = new Integer (inputValue);
		    if (iInputValue.intValue() >= 1 && iInputValue.intValue() <=  30000){ 
			node = new DefaultMutableTreeNode(new OidNode(1, ("Packet_" + iPacketNumber++), inputValue, null ) );
			iPPacketEditor.getTreeParent().add(node);
			model = new DefaultTreeModel( iPPacketEditor.getTreeParent() );
			iPPacketEditor.getTree().setModel(model);  
		    }
		}
		catch (NumberFormatException nfe){
		    errorMessage.printErrorMessage ("Integer value expected");
		}
	    }
	}
    }

    /**
     * <code>deletePacket</code> deletes packet from packetset.
     *
     */
    public void deletePacket ()
    {
        boolean bCorrectParent = false;
        node = iPPacketEditor.getSelectedTreeNode();
        if (node != null){
            MutableTreeNode parent = (MutableTreeNode) node.getParent();
            // Node has at least one parent
            if (parent != null) {
                MutableTreeNode newParent = (MutableTreeNode) parent.getParent();
                if ( newParent == null )
                    bCorrectParent = true;
            }
            if ( bCorrectParent && parent != null )
                parent.remove((MutableTreeNode)iPPacketEditor.getSelectedTreeNode());
        }
    }

    /**
     * <code>sendPacket</code> sends packet to the Packet Engine.
     *
     */
    public void sendPacket ()
    {
        XMLDocument xmldocument = new XMLDocument(iPPacketEditor.getTree());
        boolean bOk = xmldocument.createDOMDocument(iPPacketEditor.getTreeParent());
        if (bOk){
            String XMLPacket = xmldocument.documentToString();
	    if (XMLPacket != null){
		openDebugWindow();

		packetEngine = getPacketEngine();
		PacketEngine.setDebugLevel(iDebugLevel);
		packetEngine.setXMLSpec(XMLPacket);
	    }
	    else
		errorMessage.printErrorMessage("Creation of the sendable string failed");	
        }
        else
            errorMessage.printErrorMessage("Creation of the DOM document failed");
    }

    /**
     * <code>help</code>
     *
     */
    public void help ()
    {
	JOptionPane pane = new JOptionPane("Help is not implemented yet, so I hope You don't really need it...   ");
	JDialog dialog = pane.createDialog (iPPacketEditor.jMain,"Help !!!!"); 
	dialog.show();
    }
    
    /**
     * <code>about</code> tells something about this program
     *
     */
    public void about ()
    {
	String aboutText = new String ("Nipper 0.0.1\n" + "(c) VTT 2002\n");
	JOptionPane.showMessageDialog(iPPacketEditor.jMain, aboutText,
				      "About this program", JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * <code>addToTree</code> adds created protocol entity
     * to the selected packet node.
     * 
     */
    private void addToTree(Protocol proto) {
        DefaultTreeModel model = null;

        node = new DefaultMutableTreeNode(new OidNode(iProtocolNumber++, proto.getTag(), proto) );
        iPPacketEditor.getSelectedTreeNode().add(node);

        int row = iPPacketEditor.getTree().getLeadSelectionRow();

        if (iPPacketEditor.getTree().isCollapsed(row)) {
            iPPacketEditor.getTree().expandRow(row);
        }
    }

    /**
     * <code>xmlToString</code> created first XML from packet tree,
     * and then XML in converted to the String
     *
     */
    private String xmlToString ()
    {
	
        XMLDocument xmldocument = new XMLDocument(iPPacketEditor.getTree());
        boolean bDomCreated = xmldocument.createDOMDocument(iPPacketEditor.getTreeParent());
        if (bDomCreated){
            String XMLPacket = xmldocument.documentToString();
	    if (XMLPacket != null)
		return XMLPacket;
	    else
		return null;
	}
	return null;
    }

    /**
     * <code>saveStringAs</code> saves given String 
     * to the file named in the file chooser.
     *
     */
    private void saveStringAs  (String stringToSave){

	StringBuffer filePath = null;
	File dir = new File(".");
	JFileChooser fileChooser = new JFileChooser();
	PEFileFilter filter = new PEFileFilter("txt","Text file" );
	fileChooser.addChoosableFileFilter(filter);
	fileChooser.setCurrentDirectory(dir);
	    
	if(fileChooser.showSaveDialog(iPPacketEditor.jMain) != JFileChooser.APPROVE_OPTION) {
	    return;
	}
	else {
	    filePath = new StringBuffer(fileChooser.getSelectedFile().getPath()); 
	    if (fileChooser.getSelectedFile().exists()){
		JOptionPane overridePane = new JOptionPane ("Override PacketSet?");
		Object [] options = new String []{"Yes", "Cancel"};
		overridePane.setOptions(options);
		JDialog dialog = overridePane.createDialog(iPPacketEditor.jMain, "Override PacketSet?");
		dialog.show();
		Object obj = overridePane.getValue();
		//override file
		if (obj.equals ("Yes")){
		    if (filePath.substring(filePath.length()-4).equals(".txt") == false) 
			filePath.append(".txt");
		    try {
			FileWriter fileWriter = new FileWriter(filePath.toString());
			fileWriter.write(stringToSave);
			fileWriter.close();
			openedFilePath = filePath;
		    }
		    catch(Exception e) {
			errorMessage.printErrorMessage ("Saving PacketSet failed");
		    }
		} 
	    }
	    //file does not exist
	    else{ 
		
		if (filePath.substring(filePath.length()-4).equals(".txt") == false) 
		    filePath.append(".txt");
		try {
		    FileWriter fileWriter = new FileWriter(filePath.toString());
		    fileWriter.write(stringToSave);
		    fileWriter.close();
		    openedFilePath = filePath;
		}
		catch(Exception e) {
		    errorMessage.printErrorMessage ("Saving PacketSet failed");
		}
	    }//else
	} //else
    } //saveStringAs

    /**
     * <code>documentToTree</code> method converts
     * packet tree from DOM document.
     * 
     */
    private void documentToTree() {

        DefaultTreeModel model = null;	
	Node tmpNode = null;
	DefaultMutableTreeNode packetSetRoot = null;
	Protocol addedProtocol = null;

	if (document != null) {
	    Element root = document.getDocumentElement();

	    // Packetset
	    node = new DefaultMutableTreeNode(new OidNode(1, ( root.getTagName() ), null ) );
	    packetSetRoot = node;
	    iPPacketEditor.setTreeParent(packetSetRoot);
	    
	    NodeList nodeList = root.getChildNodes();
	    int length = nodeList.getLength();
	    
	    // Looping packet element
	    for (int i=0; i < length; i++) {
		// loop
		tmpNode = (Node)nodeList.item(i);
		node = new DefaultMutableTreeNode(new OidNode(1, ("Packet_" + iPacketNumber++), ((Element)tmpNode).getAttribute("stime"), null ) );
		packetSetRoot.add(node);
		NodeList siblingList = tmpNode.getChildNodes();
		int sibLength = siblingList.getLength();
		
		// Looping header elements
		for (int j=0; j < sibLength; j++) {
		    Node siblingNode = (Node) siblingList.item(j);
		    String protoName = ((Element)siblingNode).getAttribute("type");

		    // Now this is difficult... :-(
		    // Now we have to check correct protocol and install read values to it's parameters...
		    boolean bFound = false;
		    Protocol proto = null;

		    for (Enumeration e = iPPacketEditor.protocols.elements(); e.hasMoreElements();) {
			do {
			    Object o = e.nextElement();
			    if(o instanceof Protocol) {
				proto = (Protocol) o;
				break;
			    }
			} while(e.hasMoreElements());

			if ( protoName.equals(proto.getTag()) ) {
			    bFound = true;
			    break;
			}
		    }
		    
		    // We found correct protocol, YEAH!
		    if (bFound) {
			addedProtocol = (Protocol) proto.clone();

			// Now we must get parameters from XML Document
			if (siblingNode.hasChildNodes()) {
			    NodeList grandSiblingList = siblingNode.getChildNodes();

			    for (int k=0; k < grandSiblingList.getLength(); k++) {
				Node grandSiblingNode = (Node) grandSiblingList.item(k);
				
				// Now it's time to set parameter values...
				String value = grandSiblingNode.getFirstChild().getNodeValue();
				addedProtocol.setParamValue(grandSiblingNode.getNodeName(), value);
			    }
			}
		    } // if (bFound)

		    DefaultMutableTreeNode siblingTreeNode = new DefaultMutableTreeNode(new OidNode (1, protoName, addedProtocol));
		    node.add(siblingTreeNode);
		}

	    }	
	    
	    model = new DefaultTreeModel( packetSetRoot );
	    iPPacketEditor.getTree().setModel(model);        
	}
    }

    /**
     * <code>openNewTreee</code> method opens new, empty packetset tree.
     *
     * 
     */
    private void openNewTree ()
    {
	iPacketNumber = 1;
	openedFilePath = null;
        DefaultMutableTreeNode packetSetRoot  = new DefaultMutableTreeNode(new OidNode(1, "PacketSet", null));
        iPPacketEditor.setTreeParent(packetSetRoot);
        node = new DefaultMutableTreeNode(new OidNode(1, ("Packet_" + iPacketNumber++), "1", null ) );
        packetSetRoot.add(node);
        DefaultTreeModel model = new DefaultTreeModel(packetSetRoot);
        iPPacketEditor.getTree().setModel(model);     
    }


    /**
     * Returns an instance of the <code>PacketEngine</code>.
     * If engine is not instaniated yet, this method creates an instance.
     * @return a <code> PacketEngine </code> instance.
     */
    public PacketEngine getPacketEngine() {
	    if(wmo == null) {
		wmo = new WindowMessageOutput();
		wmo.init();
	    }
	    if(packetEngine == null || !packetEngine.isAlive() ) {
		packetEngine = new PacketEngine(wmo, wmo, iDebugLevel);	
		packetEngine.start();	    
	    }
	    return(packetEngine);
    }

}//class Action
