/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packeteditor;

import java.util.*;

/**
 * class <code> Protocol.java</code>
 *
 * @author  Kimmo Ahola, Titta Koskinen
 * @version 1.0
 *
 * Created on 25. keskuuta 2002, 10:00
 */

public class Protocol implements Cloneable{
    private String name;
    private String tag;
    private Vector parameters;
    private int iProtonum = 0;

    /**
     * Creates a new <code>Protocol</code> instance.
     *
     * @param _tag a <code>String</code> value
     * @param _name a <code>String</code> value
     */
    public Protocol(String _tag, String _name) {
	name = _name;
	tag = _tag;
	parameters = new Vector();
    }

    /**
     * <code>clone</code> creates and returns a copy of Protocol object.
     *
     * @return an <code>Object</code> value
     */
    public Object clone()
    {
	try {
	    Protocol proto = (Protocol)super.clone();
	    Vector newParameters = new Vector();

	    for (Enumeration e = this.parameters.elements(); e.hasMoreElements();) {
		newParameters.add( ((Parameter)e.nextElement()).clone() );
	    }
	    proto.parameters = newParameters;
	    String _name = new String(this.name);
	    String _tag = new String(this.tag);
	    proto.name = _name;
	    proto.tag = _tag;
	    return proto;
	    
	} 
	catch (CloneNotSupportedException e){
	    return null;
	}
    }

    /**
     * <code>getName</code>
     *
     * @return a <code>String</code> name of the protocol.
     */
    public String getName() {
	return name;
    }

    /**
     * <code>getTag</code>
     *
     * @return a <code>String</code> tag of the protocol.
     */
    public String getTag() {
	return tag;
    }

    /**
     * <code>getParameters</code>
     *
     * @return a <code>Vector</code> of protocol's parameters.
     */
    public Vector getParameters() {
	return parameters;
    }

    /**
     * <code>setParamValue</code> sets given name and tag for a protocol.
     *
     * @param _tag a <code>String</code> value
     * @param _value a <code>String</code> value
     */
    public void setParamValue(String _tag, String _value) {
	int i;
	Enumeration e;	
	Parameter p = null;

        for (e = parameters.elements(); e.hasMoreElements() ;) {
	    p = (Parameter) e.nextElement();

	    if ( p.getTag().equals(_tag) ) {
		p.setValue( _value);
		break;
	    }
	}	
    }

    /**
     * <code>setParameter</code> sets given name, tag and value for a protocol.
     *
     * @param _tag a <code>String</code> value
     * @param _name a <code>String</code> value
     * @param _value a <code>String</code> value
     */
    public void setParameter(String _tag, String _name, String _value) {
	int i, iFound = 0;
	Enumeration e;	
	Parameter p = null;

        for (e = parameters.elements(); e.hasMoreElements() ;) {
	    iFound = 0;
	    p = (Parameter) e.nextElement();

	    if ( p.getTag().equals(_tag) ) {
		iFound = 1;
		break;
	    }
	}	

	if (iFound != 1) {
	    p = new Parameter(_tag, _name, _value);

	    if (p == null) {
	    	System.out.println("Protocol.setParameter: Creation of new parameter failed!");
	    }

	    parameters.addElement( (Object)p );
	}
    }

}


/**
 * class <code> Parameter </code>
 *
 * @author  Kimmo Ahola, Titta Koskinen
 * @version 1.0
 *
 * Created on 25. keskuuta 2002, 10:00
 */

class Parameter implements Cloneable {
    private String pTag;
    private String pName;
    private String pValue;
   
    /**
     * Creates a new <code>Parameter</code> instance.
     *
     * @param _pTag a <code>String</code> value
     * @param _pName a <code>String</code> value
     * @param _pValue a <code>String</code> value
     */
    public Parameter(String _pTag, String _pName, String _pValue) {
	pTag = _pTag;
	pName = _pName;
	pValue = _pValue;
    }

    /**
     * <code>clone</code> creates and returns a copy of Parameter object.
     *
     * @return an <code>Object</code> value
     */
    public Object clone()
    {
	try {
	    Parameter parameter = (Parameter)super.clone();	    

	    parameter.pTag = new String(this.pTag);
	    parameter.pName = new String(this.pName);
	    parameter.pValue = new String(this.pValue);
	    return parameter;	    
	    
	} 
	catch (CloneNotSupportedException e){
	    return null;
	}
    }

    /**
     * <code>getTag</code>
     *
     * @return a <code>String</code> tag of the parameter.
     */
    public String getTag() {
	return pTag;
    }

    /**
     * <code>getName</code>
     *
     * @return a <code>String</code> name of the parameter.
     */
    public String getName() {
	return pName;
    }

    /**
     * <code>getValue</code>
     *
     * @return a <code>String</code> value of the parameter.
     */
    public String getValue() {
	return pValue;
    }

    /**
     * <code>setTag</code> sets given tag for a parameter.
     *
     * @param _pTag a <code>String</code> value
     *
     */
    public void setTag(String _pTag) {
	pTag = _pTag;
    }

    /**
     * <code>setName</code> sets given name for a parameter.
     *
     * @param _pName a <code>String</code> value
     * 
     */
    public void setName(String _pName) {
	pName = _pName;
    }

    /**
     * <code>setValue</code> sets given value for a parameter.
     *
     * @param _tag a <code>String</code> value
     * 
     */
    public void setValue(String _pValue) {
	pValue = _pValue;
    }
}
