/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packeteditor;

import packetengine.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * <code>WindowMessageOutput</code> manages debug window. 
 *
 * @author Sami Pnknen, Titta Koskinen
 * @version 1.0
 * 
 * Created on 4. heinkuuta 2002, 16:26
 */
public class WindowMessageOutput extends JFrame implements MessageOutput, ActionListener {
    
    private JTextArea outputPane;
    private Container contentPane;
    private JPanel jPanel;
    private JPanel jPanel2;
    private JButton jButtonClear;
    private JButton jButtonClose;
    private JScrollPane scrollPane;

    /**
     * Creates a new <code>WindowMessageOutput</code> instance.
     *
     */
    public WindowMessageOutput() {
	super("Nipper Debug Window");
    }

    /**
     * <code>init</code> method creates new
     * debug window.
     *
     */
    public void init() {
	
	contentPane = getContentPane();
	jPanel = new JPanel();
	jPanel2 = new JPanel();
	jPanel.setLayout(new BorderLayout());

	//Printing area
 	outputPane = new JTextArea(24, 80);
 	outputPane.setEditable(false);
 	scrollPane = new JScrollPane(outputPane, 
 					 	 JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
  						 JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
 	jPanel.add(scrollPane, BorderLayout.CENTER);
	
	//Button
	jButtonClear = new JButton("Clear");
	jPanel2.add(jButtonClear);
	jButtonClose = new JButton("Close");
	jPanel2.add(jButtonClose);
	jPanel.add(jPanel2, BorderLayout.SOUTH);
	contentPane.add(jPanel);
        jButtonClear.addActionListener(this);
	jButtonClose.addActionListener(this);
	setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	pack();
    }

    /**
     * <code>actionPerformed</code> invoked when an action occurs.
     * 
     *
     * @param evt a <code>ActionEvent</code> indicates the action to occur
     */
    public synchronized void actionPerformed (ActionEvent evt) 
    {
	Object source = evt.getSource();
	if (source == jButtonClear) {
	    outputPane.setText("");
	}
	else if (source == jButtonClose) {
	    setVisible(false);
	}
	
    }
    
    /**
     * <code>openWindow</code> opens debug window.
     * 
     *
     */
    public synchronized void openWindow () {
	show();
    }

    /**
     * <code>closeWindow</code> closes debug window.
     * 
     *
     */
    public synchronized void closeWindow () {
	setVisible(false);
    }

    /* MessageOutput methods */

    public synchronized void popup() {
	show();
    }

    /**
     * <code>print</code> method prints given message 
     * in the debug window.
     *
     * @param msg a <code>String</code> message to be printed
     */
    public synchronized void print(String msg) {
	//show();
	outputPane.append(msg);
    }

    /**
     * <code>println</code> prints given message
     * and line break
     *
     * @param msg a <code>String</code> message to be printed
     */
    public synchronized void println(String msg) {
	//show();
	outputPane.append(msg + "\n");
    }
}




