/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packeteditor;

import packetengine.*;

import java.util.*;
import java.io.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;

/**
 * <code> XMLDocument </code> parses DOM document from given file or tree.
 *
 * @author  Kimmo Ahola, Titta Koskinen
 * @version 1.0 
 *
 * Created on 27. keskuuta 2002, 16:26
 */

public class XMLDocument {
    private static JTree tree;
    private static Document document;
    private static String XMLDocument = null;
    private static Element root;

    /**
     * Creates a new <code>XMLDocument</code> instance.
     *
     * @param _tree a <code>JTree</code> value
     */
    public XMLDocument(JTree _tree) {
	tree = _tree;
    }

    /**
     * Creates a new <code>XMLDocument</code> instance.
     *
     */
    public XMLDocument() {

    }

    /**
     * <code>createDOMDocument</code> creates DOM document
     * from given file
     *
     * @param file a <code>FileInputStream</code>
     * @return a <code>Document</code> created DOM document
     */
    public Document createDOMDocument(FileInputStream file) {

	//byte[] jotain = new byte[2000];
	DocumentBuilderFactory factory =
	    DocumentBuilderFactory.newInstance();
	try {
    	    factory.setValidating(false);
	    DocumentBuilder builder =
		factory.newDocumentBuilder();

	    document = builder.parse((InputStream)file);
	    root = (Element) document.getDocumentElement();
	
	}
	catch (Exception exp) {
	    // Parser with specified options can't be built
	    return null;
	}
	return document;
    }

    /**
     * <code>createDOMDocument</code> creates DOM document from
     * referenced tree
     *
     * @param rootNode a <code>DefaultMutableTreeNode</code> refence to tree.
     * @return a <code>boolean</code> value defining was operation succeed.
     */
    public boolean createDOMDocument(DefaultMutableTreeNode rootNode) {

	DocumentBuilderFactory factory =
	    DocumentBuilderFactory.newInstance();
	try {
	    DocumentBuilder builder =
		factory.newDocumentBuilder();
	    
	    document = builder.newDocument();
	    
	    root = (Element) document.createElement(PacketEngine.getDocumentTag()); 
	    document.appendChild(root);

	    //Convert JTree to DOM document
	    if ( rootNode.getChildCount() > 0 ) {
		for (Enumeration e = rootNode.children(); e.hasMoreElements();) {
		    DefaultMutableTreeNode children = (DefaultMutableTreeNode) e.nextElement();
		    
		    OidNode oidNode = (OidNode) children.getUserObject();
		    Element node = (Element) document.createElement(PacketEngine.getPacketTag());
		    String strStime = oidNode.getSendTime();
		    if(strStime != null && !"".equals(strStime))
			node.setAttribute(PacketEngine.getPacketStimeTag(), strStime);

		    if ( children.getChildCount() > 0 ) {			
			for (Enumeration v = children.children(); v.hasMoreElements();) {
			    DefaultMutableTreeNode grandChildren = (DefaultMutableTreeNode) v.nextElement();
			    OidNode grandOidNode = (OidNode) grandChildren.getUserObject();
			    Protocol grandProto = grandOidNode.getProtocol();
			    Element grandNode= (Element) document.createElement(PacketEngine.getHeaderTag());
			    grandNode.setAttribute (PacketEngine.getHeaderTypeTag(), grandProto.getTag());

			    // Now parameters
			    for ( Enumeration s = grandProto.getParameters().elements(); s.hasMoreElements(); ) {
				Parameter param = (Parameter) s.nextElement();
				String val = param.getValue();
				if (val == null || "".equals(val))
				    continue;

				// For example IPdest, ihl ...
				Element grandElementNode = (Element) document.createElement(param.getTag());
				// and here its value

				Text grandTextNode = (Text) document.createTextNode(param.getValue());
				grandElementNode.appendChild(grandTextNode);

				// Adding parameter to protocol
				grandNode.appendChild(grandElementNode);
			    }
			    // Adding protocol to packet
			    node.appendChild(grandNode);
			}
		    }
		    // Adding Packet to packetset
		    root.appendChild(node);
		}
	    }
	    
	} catch (ParserConfigurationException pce) {
	    // Parser with specified options can't be built
	    return false;
	}
	return true;
    }
    
    /**
     * <code>documentToString</code> convert string from
     * created DOM document
     *
     * @return a <code>String</code> value that was covcerted
     */
    public String documentToString() {

	    try {
		TransformerFactory tf = TransformerFactory.newInstance();
		if(tf.getFeature(DOMSource.FEATURE) && tf.getFeature(StreamResult.FEATURE)) {
		    StringWriter strw = new StringWriter();
		    DOMSource src = new DOMSource(root);
		    StreamResult dst = new StreamResult(strw);
		    Transformer t = tf.newTransformer();
		    t.transform(src, dst);
		    XMLDocument = strw.toString();
		}
	    }
	    catch(Exception exp) {
		exp.printStackTrace();
		return null;
	    }   
	    return XMLDocument;
    }

}



