/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine;

public abstract class NetworkInterface {


    /**
     * Return a type tag for the interface.
     * @return A tag identifying the implementing class.
     */
    public static String getIFTag() {
	return("");
    }

    /**
     * Set parameters for the interface.
     * @param params A string containing the parameters, 
     *               such as an IP address or port number.
     * @return true if parameters were successfully set,
     *         false otherwise.
     */
    public abstract boolean setParams(String params);

    /**
     * Activate interface.
     * @return true if success,
     *         false otherwise
     */
    public abstract boolean ifup();

    /**
     * Bring down the interface.
     * @return true if success,
     *         false otherwise
     */
    public abstract boolean ifdown();

    /**
     * Send an IP packet to network.
     * @param packet An array of bytes containing the packet
     * @return true if the packet was successfully sent to network,
     *         false otherwise.
     */
    public abstract boolean sendData(byte[] packet);

}
