/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine;

import packetengine.header.*;
import java.util.*;

class Packet {
    private LinkedList headerList;
    private byte[] packetBuffer;
    private int stime;

    Packet() {
	headerList = new LinkedList();
	packetBuffer = null;
	stime = 0;
    }

    void setSTime(int stime) {
	this.stime = stime;
    }

    int getSTime() {
	return(stime);
    }

    void add(Header h) {
	headerList.add(h);
    }

    void build() {

	byte[] buildBuffer = new byte[65536];
	int length = buildBuffer.length;
	int ptr = length;

	Header h, h2;

	PacketUtils.debugln(3, "Packet.build() - Building packets");

	/* First loop: Build and checksum headers */
	for(int i=headerList.size()-1; i>=0; i--) {
	    h = (Header) headerList.get(i);

	    PacketUtils.debugln(3, "Packet.build() loop #1 - h.getHeaderType() = " + h.getHeaderType());
	    PacketUtils.debugln(5, "length = " + h.getLength());

	    ptr -= h.build(buildBuffer, ptr, length-ptr, headerList, i);
	    ptr -= h.copyBytes(buildBuffer, ptr - h.getLength(), h.getLength());
	}

	/* Calculate size of the buffer */
	int total_length = length - ptr;
	packetBuffer = new byte[total_length];
	ptr = total_length;
	PacketUtils.debugln(3, "Packet.build() - total_length = " + total_length);

	/* Second Loop: Copy bytes to output buffer */
	for(int i=headerList.size()-1; i>=0; i--) {
	    h = (Header) headerList.get(i);
	    PacketUtils.debugln(3, "Packet.build() loop #2 - h.getHeaderType() = " + h.getHeaderType());
	    ptr -= h.copyBytes(packetBuffer, ptr - h.getLength(), h.getLength());
	}

	PacketUtils.debugln(3, "Packet.build() - Packets successfully built");
    }

    byte[] getBytes() {
	return(packetBuffer);
    }
}

