/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv4_OPT_SEC extends Header implements IPv4Option {

    public IPv4_OPT_SEC() {
	super();

        buffer = new byte[11];
        start = 0;
        end = 11;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_SEC, NAME_SEC);
	h.put(TAG_COMP, NAME_COMP);
	h.put(TAG_HAND, NAME_HAND);
	h.put(TAG_TCC, NAME_TCC);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = (byte) (130 & 0xff);
	buffer[start+1] = (byte) (11 & 0xff);

	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_SEC.equals(tag)) {
            PacketUtils.debugln(5, TAG_SEC + ": " + new String(ch, offset, length));
            setSecurity(PacketUtils.parseInt(new String(ch, offset, length)));
        }
        else if(TAG_COMP.equals(tag)) {
            PacketUtils.debugln(5, TAG_COMP + ": " + new String(ch, offset, length));
            setCompartments(PacketUtils.parseInt(new String(ch, offset, length)));
	}
        else if(TAG_HAND.equals(tag)) {
            PacketUtils.debugln(5, TAG_HAND + ": " + new String(ch, offset, length));
            setHandling(PacketUtils.parseInt(new String(ch, offset, length)));
        }
        else if(TAG_TCC.equals(tag)) {
            PacketUtils.debugln(5, TAG_TCC + ": " + new String(ch, offset, length));
            setTCC(PacketUtils.parseInt(new String(ch, offset, length)));
        }
    }

    /* IPv4_OPT_SEC methods */

    public void setSecurity(int val) {
        buffer[start+2] = (byte) ((val & 0xff00) >>> 8);
        buffer[start+3] = (byte) (val & 0x00ff);
    }

    public void setCompartments(int val) {
        buffer[start+4] = (byte) ((val & 0xff00) >>> 8);
        buffer[start+5] = (byte) (val & 0x00ff);
    }

    public void setHandling(int val) {
        buffer[start+6] = (byte) ((val & 0xff00) >>> 8);
        buffer[start+7] = (byte) (val & 0x00ff);
    }

    public void setTCC(int val) {
        buffer[start+8] = (byte) ((val & 0xff0000) >>> 16);
        buffer[start+9] = (byte) ((val & 0x00ff00) >>> 8);
        buffer[start+10] = (byte) (val & 0x0000ff);
    }

    /* private IPv4_OPT_SEC methods */

    /* name */
    private static final String TAG_NAME = "IPv4_OPT_SEC";
    private static final String NAME_NAME = "IPv4 Option Security (SEC)";

    /* tags */
    private static final String TAG_SEC = "s";
    private static final String NAME_SEC = "Security";

    private static final String TAG_COMP = "c";
    private static final String NAME_COMP = "Compartments";

    private static final String TAG_HAND = "h";
    private static final String NAME_HAND = "Handling Restrictions";

    private static final String TAG_TCC = "tcc";
    private static final String NAME_TCC = "Transmission Control Code";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

