/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv4_OPT_SSRR extends Header implements IPv4Option {

    private int addr_ptr;
    
    public IPv4_OPT_SSRR() {
	super();

        buffer = new byte[40];
        start = 0;
        end = 3;
	addr_ptr = 3;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_LENGTH, NAME_LENGTH);
	h.put(TAG_PTR, NAME_PTR);
	h.put(TAG_RDATA, NAME_RDATA);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = (byte) (137 & 0xff);
	if(!FLAG_LENGTH) {
	    end = addr_ptr;
	    setLength((byte) (getLength() & 0xff));
	}
	if(!FLAG_PTR)
	    setPointer((byte) 4);

	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
	    FLAG_LENGTH = false;
            setLength((byte) (PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_LENGTH = true;
        }
        else if(TAG_PTR.equals(tag)) {
            PacketUtils.debugln(5, TAG_PTR + ": " + new String(ch, offset, length));
	    FLAG_PTR = false;
            setPointer((byte) (PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_PTR = true;
	}
        else if(TAG_RDATA.equals(tag)) {
            PacketUtils.debugln(5, TAG_RDATA + ": " + new String(ch, offset, length));
            StringTokenizer stok = new StringTokenizer(new String(ch, offset, length), ",");
            while(stok.hasMoreTokens()) {
                try {
                    setRdata(Inet4Address.getByName(stok.nextToken().trim()).getAddress());
                }
                catch(UnknownHostException e) {
		    PacketUtils.errdebugln(e.toString());
		    throw new SAXException(e);
                }
            }
        }
    }

    /* IPv4_OPT_SSRR methods */

    public void setLength(byte val) {
        buffer[start+1] = val;
	end = start + val;
    }

    public void setPointer(byte val) {
        buffer[start+2] = val;
    }

    public void setRdata(Inet4Address addr) {
	if(addr != null)
	    setRdata(addr.getAddress());
    }

    public void setRdata(byte[] addr) {
        int i, j = end;

	if(end == 3)
	    j = buffer.length;
	
        if(addr_ptr+4 >= buffer.length)
            return;

        for(i=0; i<4 && addr_ptr+i<j; i++)
            buffer[addr_ptr+i] = addr[i];

        addr_ptr += i;
    }

    /* private IPv4_OPT_SSRR methods */

    /* name */
    private static final String TAG_NAME = "IPv4_OPT_SSRR";
    private static final String NAME_NAME = "IPv4 Option Strict Source and Record Route (SSRR)";

    /* tags */

    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Length";
    private boolean FLAG_LENGTH = false;

    private static final String TAG_PTR = "ptr";
    private static final String NAME_PTR = "Pointer";
    private boolean FLAG_PTR = false;

    private static final String TAG_RDATA = "addr";
    private static final String NAME_RDATA = "Address";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

