/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv4_OPT_TS extends Header implements IPv4Option {

    private int aptr;
    private int tptr;

    public IPv4_OPT_TS() {
	super();

        buffer = new byte[40];
        start = 0;
        end = 4;

	aptr = 4;
	tptr = 0;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_LENGTH, NAME_LENGTH);
	h.put(TAG_PTR, NAME_PTR);
	h.put(TAG_OFLW, NAME_OFLW);
	h.put(TAG_FLAGS, NAME_FLAGS);
	h.put(TAG_ADDR, NAME_ADDR);
	h.put(TAG_STAMP, NAME_STAMP);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = (byte) (68 & 0xff);
	if(!FLAG_LENGTH)
	    setLength((byte) (getLength() & 0xff));
	if(!FLAG_PTR)
	    setPointer((byte) 5);

	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
            setLength((byte) (PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_LENGTH = true;
        }
        else if(TAG_PTR.equals(tag)) {
            PacketUtils.debugln(5, TAG_PTR + ": " + new String(ch, offset, length));
            setPointer((byte) (PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_PTR = true;
	}
        else if(TAG_OFLW.equals(tag)) {
            PacketUtils.debugln(5, TAG_OFLW + ": " + new String(ch, offset, length));
            setOverflow((byte) (PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
        else if(TAG_FLAGS.equals(tag)) {
            PacketUtils.debugln(5, TAG_FLAGS + ": " + new String(ch, offset, length));
            setFlags((byte) (PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	}
        else if(TAG_ADDR.equals(tag)) {
            PacketUtils.debugln(5, TAG_ADDR + ": " + new String(ch, offset, length));
	    StringTokenizer stok = new StringTokenizer(new String(ch, offset, length), ",");
	    while(stok.hasMoreTokens()) {
		try {
		    setAddress(Inet4Address.getByName(stok.nextToken().trim()).getAddress());
		}
		catch(UnknownHostException e) {
		    PacketUtils.errdebugln(e.toString());
		    throw new SAXException(e);
		}
	    }
	}
	else if(TAG_STAMP.equals(tag)) {
            PacketUtils.debugln(5, TAG_STAMP + ": " + new String(ch, offset, length));
	    StringTokenizer stok = new StringTokenizer(new String(ch, offset, length), ",");
	    while(stok.hasMoreTokens())
		setTimestamp(PacketUtils.parseInt(stok.nextToken().trim()));
	}
    }

    /* IPv4_OPT_TS methods */

    public void setLength(byte val) {
        buffer[start+1] = val;
    }

    public void setPointer(byte val) {
        buffer[start+2] = val;
    }

    public void setOverflow(byte val) {
	buffer[start+3] = (byte) (( val & 0x0f ) << 4 | (buffer[start+3] & 0x0f));
    }

    public void setFlags(byte val) {
	buffer[start+3] = (byte) (( val & 0x0f ) | (buffer[start+3] & 0xf0));
    }

    public void setAddress(Inet4Address addr) {
	if(addr != null)
	    setAddress(addr.getAddress());
    }

    public void setAddress(byte[] addr) {
	if(getFlags() == 0)
	    return;
	
	if(aptr >= buffer.length)
	    return;
	
	for(int i=0; i<4 && aptr+i<buffer.length; i++)
	    buffer[aptr+i] = addr[i];
	
	aptr += 8;
	end = Math.max(end + 4, end);
    }

    public void setTimestamp(int stamp) {
	int hop;

	if(tptr+4 >= buffer.length)
	    return;

	if(getFlags() == 0)
	    hop = 4;
	else 
	    hop = 8;

	if(tptr == 0)
	    tptr = hop;
	
	buffer[tptr] =   (byte) ((stamp & 0xff000000) >>> 24);
	buffer[tptr+1] = (byte) ((stamp & 0x00ff0000) >>> 16);
	buffer[tptr+2] = (byte) ((stamp & 0x0000ff00) >>> 8);
	buffer[tptr+3] = (byte) ((stamp & 0x000000ff));

	tptr += hop;
	end = Math.max(end + 4, end);
    }

    /* private IPv4_OPT_TS methods */

    private int getFlags() {
	return(buffer[start+3] & 0x0f);
    }

    /* name */
    private static final String TAG_NAME = "IPv4_OPT_TS";
    private static final String NAME_NAME = "IPv4 Option Internet Timestamp (TS)";

    /* tags */

    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Length";
    private boolean FLAG_LENGTH = false;

    private static final String TAG_PTR = "ptr";
    private static final String NAME_PTR = "Pointer";
    private boolean FLAG_PTR = false;

    private static final String TAG_OFLW = "oflw";
    private static final String NAME_OFLW = "Overflow";

    private static final String TAG_FLAGS = "flg";
    private static final String NAME_FLAGS = "Flags";

    private static final String TAG_ADDR = "addr";
    private static final String NAME_ADDR = "Address";

    private static final String TAG_STAMP = "stamp";
    private static final String NAME_STAMP = "Timestamp";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

