/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class IPv6 extends Header implements NextHeader {

    public IPv6() {
	super();

	/* create buffer */
	buffer = new byte[40];
	start = 0;
	end = 40;

	setVersion();
    }

    public IPv6(byte[] buffer, int offset) {
	super();
	
	/* set buffer */
	this.buffer = buffer;
	start = offset;
	end = start + 40;
    }

    /* Header methods */

    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public String getHeaderType() {
        return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
        h.put(TAG_VERSION, NAME_VERSION);
        h.put(TAG_PRIORITY, NAME_PRIORITY);
        h.put(TAG_FLOWLABEL, NAME_FLOWLABEL);
        h.put(TAG_LENGTH, NAME_LENGTH);
        h.put(TAG_NEXTHEADER, NAME_NEXTHEADER);
        h.put(TAG_HOPLIMIT, NAME_HOPLIMIT);
        h.put(TAG_SADDR, NAME_SADDR);
        h.put(TAG_DADDR, NAME_DADDR);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	Header h = null, h2 = null;
	int i;
	int ptr = 0;

	/* Handle IPv6 and options */
	//else if(h.getClass().getName().startsWith(Header.PACKAGE_PREFIX + "." + "IPv6")) {	    
	//if(h instanceof IPv6 || h instanceof IPv6Option || h instanceof IPv6Ext) {
	
	for(i=index+1; i<headerList.size(); i++) {
	    h = (Header) headerList.get(i);
	    if(h != null && (h instanceof IPv6Ext || h instanceof IPv6Option || h instanceof IPv6SubOption));
	    else
		break;
	}

	for(i--; i>=index; i--) {
	    h = (Header) headerList.get(i);

	    /* Set NextHeader fields */
	    if(h != null && h instanceof NextHeader) {
		h2 = null;
		for(int j=i+1; j<headerList.size(); j++) {
		    h2 = (Header) headerList.get(j);
		    if(h2 != null && h2.getProtocolID() != Header.UNDEFINED_PROTOCOL)
			break;
		}
		if(h2 != null)
		    ((NextHeader) h).setNextHeader(h2.getProtocolID());
	    }
	    
	    /* Set Extension Header Lengths */
	    if(h != null && h instanceof IPv6Ext) {
		int len = 0;
		h2 = null;
		for(int j=i+1; j<headerList.size(); j++) {
		    h2 = (Header) headerList.get(j);
		    if(h2 != null && (h2 instanceof IPv6Option || h2 instanceof IPv6SubOption))
			len += h2.getLength();
		    else
			break;
		}
		len += h.getLength();
		((IPv6Ext) h).setLength((byte) (Math.ceil( (double) len / (double) 8) - 1));
	    }
	    
	    /* Set Option Lengths */
	    if(h != null && h instanceof IPv6Option) {
		int len = 0;
		h2 = null;
		for(int j=i+1; j<headerList.size(); j++) {
		    h2 = (Header) headerList.get(j);
		    if(h2 != null && (h2 instanceof IPv6SubOption))
			len += h2.getLength();
		    else
			break;
		}
		((IPv6Option) h).setLength((byte) len);
	    }
	}

	/* Set IPv6 Payload Length */
	setPayloadLength(length - ptr);

	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
        if(TAG_VERSION.equals(tag)) {
            PacketUtils.debugln(5, TAG_VERSION + ": " + new String(ch, offset, length));
	    FLAG_VERSION = false;
            setVersion((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_VERSION = true;
        }
        else if(TAG_PRIORITY.equals(tag)) {
            PacketUtils.debugln(5, TAG_PRIORITY + ": " + new String(ch, offset, length));
            setPriority((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
        else if(TAG_FLOWLABEL.equals(tag)) {
            PacketUtils.debugln(5, TAG_FLOWLABEL + ": " + new String(ch, offset, length));
            setFlowLabel(PacketUtils.parseInt(new String(ch, offset, length)));
	}
        else if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
            setPayloadLength(PacketUtils.parseInt(new String(ch, offset, length)));
	}
        else if(TAG_NEXTHEADER.equals(tag)) {
            PacketUtils.debugln(5, TAG_HEADER + ": " + new String(ch, offset, length));
	    FLAG_NEXTHEADER = false;
            setNextHeader((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_NEXTHEADER = true;
        }
        else if(TAG_HOPLIMIT.equals(tag)) {
            PacketUtils.debugln(5, TAG_HOPLIMIT + ": " + new String(ch, offset, length));
            setHopLimit((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
        else if(TAG_SADDR.equals(tag)) {
            PacketUtils.debugln(5, TAG_SADDR + ": " + new String(ch, offset, length));
            try {
                setSourceAddress(Inet6Address.getByName(new String(ch, offset, length)).getAddress());
            }
            catch(UnknownHostException e) {
		PacketUtils.errdebugln(e.toString());
		throw new SAXException(e);
            }
        }
        else if(TAG_DADDR.equals(tag)) {
            PacketUtils.debugln(5, TAG_DADDR + ": " + new String(ch, offset, length));
            try {
                setDestinationAddress(Inet6Address.getByName(new String(ch, offset, length)).getAddress());
            }
            catch(UnknownHostException e) {
		PacketUtils.errdebugln(e.toString());
		throw new SAXException(e);
            }
        }
    }

    /* IPv6 methods */

    public void setVersion() {
	if(!FLAG_VERSION)
	    buffer[start] = (byte) (0x60 | (buffer[start] & 0x0f));
    }

    public void setVersion(byte version) {
	if(!FLAG_VERSION)
	    buffer[start] = (byte) (((version & 0x0f) << 4) | (buffer[start] & 0x0f));
    }
    
    public void setPriority(byte priority) {
	buffer[start] = (byte) ((priority & 0x0f) | (buffer[start] & 0xf0));
    }

    public void setFlowLabel(int flowID) {
	PacketUtils.setU24(buffer, start+1, flowID & 0x00ff0000);
    }

    public void setPayloadLength(int length) {
	PacketUtils.setU16(buffer, start+4, length);
    }

    public void setNextHeader(byte headerType) {
	if(!FLAG_NEXTHEADER)
	    PacketUtils.setU8(buffer, start+6, headerType);
    }

    public void setHopLimit(byte hopLimit) {
	PacketUtils.setU8(buffer, start+7, hopLimit);
    }

    public void setSourceAddress(Inet6Address address) {
	if(address != null)
	    setSourceAddress(address.getAddress());
    }

    public void setSourceAddress(byte[] address) {
	if(address.length < 16)
	    return;

	for(int i=0; i<16; i++)
	    buffer[start+8+i] = address[i];
    }

    public void setDestinationAddress(Inet6Address address) {
	if(address != null)
	    setDestinationAddress(address.getAddress());
    }

    public void setDestinationAddress(byte[] address) {
	if(address.length < 16)
	    return;

	for(int i=0; i<16; i++)
	    buffer[start+24+i] = address[i];
    }

    public byte getPriority() {
	return getPriority(buffer, start, end-start);
    }

    public int getPayloadLength() {
	return getPayloadLength(buffer, start, end-start);
    }

    public byte getNextHeader() {
	return getNextHeader(buffer, start, end-start);
    }

    public byte getHopLimit() {
	return getHopLimit(buffer, start, end-start);
    }

    public Inet6Address getSourceAddress() {
	return getSourceAddress(buffer, start, end-start);
    }

    public Inet6Address getDestinationAddress() {
	return getDestinationAddress(buffer, start, end-start);
    }

    /* static public methods */

    public static boolean identify(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(false);
        if((data[offset] & 0xf0) == 0x60)
            return(true);
        return(false);
    }

    public static byte getPriority(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(-1);
        return((byte) (data[offset] & 0x0f));
    }

    public static int getFlowLabel(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(-1);
        return(PacketUtils.getU24(data, offset));
    }

    public static int getPayloadLength(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(-1);
	return(PacketUtils.getU16(data, offset+4));
    }

    public static byte getNextHeader(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(-1);
	return(data[offset+6]);
    }

    public static byte getHopLimit(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(-1);
	return(data[offset+7]);
    }

    public static Inet6Address getSourceAddress(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(null);
        byte[] addr = new byte[16];
        System.arraycopy(data, offset+8, addr, 0, 16);
        try {
            return((Inet6Address) Inet6Address.getByAddress(addr));
        }
        catch(UnknownHostException exp) {
        }
        return(null);
    }

    public static Inet6Address getDestinationAddress(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 40)
            return(null);
        byte[] addr = new byte[16];
        System.arraycopy(data, offset+24, addr, 0, 16);
        try {
            return((Inet6Address) Inet6Address.getByAddress(addr));
        }
        catch(UnknownHostException exp) {
        }
        return(null);
    }
    
    public static int getTransportHeaderOffset(byte[] data, int offset, int length) {
	return(getTransportHeader(data, offset, length, false));
    }

    public static byte getTransportHeaderID(byte[] data, int offset, int length) {
	int id = getTransportHeader(data, offset, length, true);
	if(id < 0)
	    return(Header.UNDEFINED_PROTOCOL);
	return((byte) (id & 0xff));
    }

    /* private methods */

    private static int getTransportHeader(byte[] data, int offset, int length, boolean return_id) {
        if(data == null || offset < 0 || length < 40 || data.length < offset + length)
            return(-1);

	int nh = IPv6.getNextHeader(data, offset, length);
	int hlen = 40;
	int hoff = offset + hlen;

	while(nh != TCP.PROTOCOL_ID && nh != UDP.PROTOCOL_ID && nh != Header.UNDEFINED_PROTOCOL) {
	    if(nh == Header.NO_NEXTHEADER) {
		hoff = -1;
		break;
	    }
	    else if(nh == IPv6_HOP.PROTOCOL_ID) {
		nh = IPv6_HOP.getNextHeader(data, hoff, length - hlen);
		hlen = (IPv6_HOP.getLength(data, hoff, length - hlen) + 1) * 8;
		hoff += hlen;
	    }
	    else if(nh == IPv6_ROUTING.PROTOCOL_ID) {
		nh = IPv6_ROUTING.getNextHeader(data, hoff, length - hlen);
		hlen = (IPv6_ROUTING.getLength(data, hoff, length - hlen) + 1) * 8;
		hoff += hlen;
	    }
	    else if(nh == IPv6_FRAGMENT.PROTOCOL_ID) {
		nh = IPv6_FRAGMENT.getNextHeader(data, hoff, length - hlen);
		hlen = (IPv6_FRAGMENT.getLength(data, hoff, length - hlen) + 1) * 8;
		hoff += hlen;
	    }
	    else if(nh == IPv6_DESTINATION.PROTOCOL_ID) {
		nh = IPv6_DESTINATION.getNextHeader(data, hoff, length - hlen);
		hlen = (IPv6_DESTINATION.getLength(data, hoff, length - hlen) + 1) * 8;
		hoff += hlen;
	    }
	    else
		nh = Header.UNDEFINED_PROTOCOL;
	    
	    if(hlen < 0) {
		nh = Header.UNDEFINED_PROTOCOL;
		hoff = -1;
		break;
	    }
	}
	
	if(return_id)
	    return(nh);
	else
	    return(hoff);
    }

    /* name */
    private static final String TAG_NAME = "IPv6";
    private static final String NAME_NAME = "IPv6";

    /* tags */
    private static final String TAG_VERSION = "version";
    private static final String NAME_VERSION = "Version";
    private boolean FLAG_VERSION = false;

    private static final String TAG_PRIORITY = "priority";
    private static final String NAME_PRIORITY = "Priority";

    private static final String TAG_FLOWLABEL = "flowlabel";
    private static final String NAME_FLOWLABEL = "Flowlabel";

    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Payload Length";

    private static final String TAG_NEXTHEADER = "nextheader";
    private static final String NAME_NEXTHEADER = "Next Header";
    private boolean FLAG_NEXTHEADER = false;

    private static final String TAG_HOPLIMIT = "hoplimit";
    private static final String NAME_HOPLIMIT = "Hop Limit";

    private static final String TAG_SADDR = "srcaddr";
    private static final String NAME_SADDR = "Source Address";

    private static final String TAG_DADDR = "dstaddr";
    private static final String NAME_DADDR = "Destination Address";

    /* protocol id */
    public static final byte PROTOCOL_ID = (byte) 4;
}


