/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv6_FRAGMENT extends Header implements NextHeader, IPv6Ext {

    public IPv6_FRAGMENT() {
	super();

        parent = null;
        parser = null;
        tag = null;

        buffer = new byte[8];
        start = 0;
        end = 8;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_NEXTHEADER, NAME_NEXTHEADER);
	h.put(TAG_FRAGOFFSET, NAME_FRAGOFFSET);
	h.put(TAG_MF, NAME_MF);
	h.put(TAG_ID, NAME_ID);
        return(h);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_NEXTHEADER.equals(tag)) {
            PacketUtils.debugln(5, TAG_NEXTHEADER + ": " + new String(ch, offset, length));
	    FLAG_NEXTHEADER = false;
            setNextHeader((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_NEXTHEADER = true;
        }
	else if(TAG_FRAGOFFSET.equals(tag)) {
            PacketUtils.debugln(5, TAG_FRAGOFFSET + ": " + new String(ch, offset, length));
            setFragmentOffset(PacketUtils.parseInt(new String(ch, offset, length)));
        }
	else if(TAG_MF.equals(tag)) {
            PacketUtils.debugln(5, TAG_MF + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setMF(true);
            else
                setMF(false);
        }
	else if(TAG_ID.equals(tag)) {
            PacketUtils.debugln(5, TAG_ID + ": " + new String(ch, offset, length));
            setIdentification(PacketUtils.parseInt(new String(ch, offset, length)));
        }
    }

    /* IPv6_FRAGMENT methods */

    public void setNextHeader(byte headerType) {
	if(!FLAG_NEXTHEADER)
	    buffer[start] = headerType;
    }

    public void setLength(byte length) {
    }

    public void setFragmentOffset(int offset) {
	buffer[start+2] = (byte) ((offset & 0x1fe0) >>> 5);
	buffer[start+3] = (byte) (((offset & 0x1f) << 3) | (buffer[start+3] & 0x07));
    }

    public void setMF(boolean mf) {
	if(mf)
            buffer[start+3] = (byte) (buffer[start+3] | 0x01);
        else
            buffer[start+3] = (byte) (buffer[start+3] | 0x01);
    }

    public void setIdentification(int id) {
	buffer[start+4] = (byte) ((id & 0xff000000) >>> 24);
	buffer[start+5] = (byte) ((id & 0x00ff0000) >>> 16);
	buffer[start+6] = (byte) ((id & 0x0000ff00) >>> 8);
	buffer[start+7] = (byte) ((id & 0x000000ff));
    }

    /* static public methods */

    public static byte getNextHeader(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length <= 2 || data.length < offset + length)
            return(-1);
        return(data[offset]);
    }

    public static int getLength(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length <= 2 || data.length < offset + length)
            return(-1);
        return(0);
    }

    /* private IPv6_FRAGMENT methods */

    /* name */
    private static final String TAG_NAME = "IPv6_FRAGMENT";
    private static final String NAME_NAME = "IPv6 Fragment Header";

    /* tags */
    private static final String TAG_NEXTHEADER = "nextheader";
    private static final String NAME_NEXTHEADER = "Next Header";
    private boolean FLAG_NEXTHEADER = false;

    private static final String TAG_FRAGOFFSET = "fragoffset";
    private static final String NAME_FRAGOFFSET = "Fragment Offset";

    private static final String TAG_MF = "mf";
    private static final String NAME_MF = "More Fragments";

    private static final String TAG_ID = "id";
    private static final String NAME_ID = "Identification";

    /* protocol id */
    public static final byte PROTOCOL_ID = 44;
}

