/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv6_OPT_DESTINATION_BINDING_ACK extends Header implements IPv6Option {

    public IPv6_OPT_DESTINATION_BINDING_ACK() {
	super();

        buffer = new byte[13];
        start = 0;
        end = 13;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_LENGTH, NAME_LENGTH);
	h.put(TAG_STATUS, NAME_STATUS);
	h.put(TAG_SEQ, NAME_SEQ);
	h.put(TAG_LIFETIME, NAME_LIFETIME);
	h.put(TAG_REFRESH, NAME_REFRESH);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = (byte) 7;
	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
            FLAG_LENGTH = false;
            setLength((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
            FLAG_LENGTH = true;
        }
	else if(TAG_STATUS.equals(tag)) {
            PacketUtils.debugln(5, TAG_STATUS + ": " + new String(ch, offset, length));
            setStatus((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
	else if(TAG_SEQ.equals(tag)) {
            PacketUtils.debugln(5, TAG_SEQ + ": " + new String(ch, offset, length));
            setSequence((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
	else if(TAG_LIFETIME.equals(tag)) {
            PacketUtils.debugln(5, TAG_LIFETIME + ": " + new String(ch, offset, length));
            setLifetime(PacketUtils.parseInt(new String(ch, offset, length)));
        }
	else if(TAG_REFRESH.equals(tag)) {
            PacketUtils.debugln(5, TAG_REFRESH + ": " + new String(ch, offset, length));
            setRefresh(PacketUtils.parseInt(new String(ch, offset, length)));
        }
    }

    /* IPv6_OPT_DESTINATION_BINDING_UPDATE methods */

    public void setLength(byte length) {
	buffer[start+1] = length;
    }

    public void setStatus(byte status) {
	buffer[start+2] = status;
    }

    public void setSequence(byte seq) {
	buffer[start+4] = seq;
    }

    public void setLifetime(int lifetime) {
	buffer[start+5] = (byte) ((lifetime & 0xff000000) >>> 24);
	buffer[start+5] = (byte) ((lifetime & 0x00ff0000) >>> 16);
	buffer[start+5] = (byte) ((lifetime & 0x0000ff00) >>> 8);
	buffer[start+5] = (byte) ((lifetime & 0x000000ff));
    }

    public void setRefresh(int refresh) {
	buffer[start+9] = (byte) ((refresh & 0xff000000) >>> 24);
	buffer[start+9] = (byte) ((refresh & 0x00ff0000) >>> 16);
	buffer[start+9] = (byte) ((refresh & 0x0000ff00) >>> 8);
	buffer[start+9] = (byte) ((refresh & 0x000000ff));
    }

    /* private IPv6_OPT_DESTINATION_BINDING_ACK methods */

    /* name */
    private static final String TAG_NAME = "IPv6_OPT_DESTINATION_BINDING_ACK";
    private static final String NAME_NAME = "IPv6 Destination Option Binding Acknowledgement";

    /* tags */
    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Length";
    private boolean FLAG_LENGTH = false;

    private static final String TAG_STATUS = "status";
    private static final String NAME_STATUS = "Status";

    private static final String TAG_SEQ = "seq";
    private static final String NAME_SEQ = "Sequence";

    private static final String TAG_LIFETIME = "lifetime";
    private static final String NAME_LIFETIME = "Lifetime";

    private static final String TAG_REFRESH = "refresh";
    private static final String NAME_REFRESH = "Refresh";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

