/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv6_OPT_DESTINATION_BINDING_UPDATE extends Header implements IPv6Option {

    public IPv6_OPT_DESTINATION_BINDING_UPDATE() {
	super();

        buffer = new byte[10];
        start = 0;
        end = 10;
    }

    /* Header methods */

    public String getHeaderType() {
        return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_LENGTH, NAME_LENGTH);
	h.put(TAG_ACK, NAME_ACK);
	h.put(TAG_HOME, NAME_HOME);
	h.put(TAG_SINGLE, NAME_SINGLE);
	h.put(TAG_DUP, NAME_DUP);
	h.put(TAG_SEQ, NAME_SEQ);
	h.put(TAG_LIFETIME, NAME_LIFETIME);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = (byte) 198;
	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
        if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
            FLAG_LENGTH = false;
            setLength((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
            FLAG_LENGTH = true;
        }
	else if(TAG_ACK.equals(tag)) {
            PacketUtils.debugln(5, TAG_ACK + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setACK(true);
            else
                setACK(false);
        }
	else if(TAG_HOME.equals(tag)) {
            PacketUtils.debugln(5, TAG_HOME + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setHomeRegistration(true);
            else
                setHomeRegistration(false);
        }
	else if(TAG_SINGLE.equals(tag)) {
            PacketUtils.debugln(5, TAG_SINGLE + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setSingleAddress(true);
            else
                setSingleAddress(false);
        }
	else if(TAG_DUP.equals(tag)) {
            PacketUtils.debugln(5, TAG_DUP + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setDuplicateAddress(true);
            else
                setDuplicateAddress(false);
        }
	else if(TAG_SEQ.equals(tag)) {
            PacketUtils.debugln(5, TAG_SEQ + ": " + new String(ch, offset, length));
            setSequence((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
	else if(TAG_LIFETIME.equals(tag)) {
            PacketUtils.debugln(5, TAG_LIFETIME + ": " + new String(ch, offset, length));
            setLifetime(PacketUtils.parseInt(new String(ch, offset, length)));
        }
    }

    /* IPv6_OPT_DESTINATION_BINDING_UPDATE methods */

    public void setLength(byte length) {
	buffer[start+1] = length;
    }

    public void setACK(boolean ack) {
        if(ack)
            buffer[start+2] = (byte) (buffer[start+6] | 0x80);
        else
            buffer[start+2] = (byte) (buffer[start+6] & 0x7f);
    }

    public void setHomeRegistration(boolean h) {
        if(h)
            buffer[start+2] = (byte) (buffer[start+6] | 0x40);
        else
            buffer[start+2] = (byte) (buffer[start+6] & 0xbf);
    }

    public void setSingleAddress(boolean s) {
        if(s)
            buffer[start+2] = (byte) (buffer[start+6] | 0x20);
        else
            buffer[start+2] = (byte) (buffer[start+6] & 0xdf);
    }

    public void setDuplicateAddress(boolean d) {
        if(d)
            buffer[start+2] = (byte) (buffer[start+6] | 0x10);
        else
            buffer[start+2] = (byte) (buffer[start+6] & 0xef);
    }

    public void setSequence(byte seq) {
	buffer[start+5] = seq;
    }

    public void setLifetime(int lifetime) {
	buffer[start+6] = (byte) ((lifetime & 0xff000000) >>> 24);
	buffer[start+6] = (byte) ((lifetime & 0x00ff0000) >>> 16);
	buffer[start+6] = (byte) ((lifetime & 0x0000ff00) >>> 8);
	buffer[start+6] = (byte) ((lifetime & 0x000000ff));
    }

    /* private IPv6_OPT_DESTINATION_BINDING_UPDATE methods */

    /* name */
    private static final String TAG_NAME = "IPv6_OPT_DESTINATION_BINDING_UPDATE";
    private static final String NAME_NAME = "IPv6 Destination Option Binding Update";

    /* tags */
    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Length";
    private boolean FLAG_LENGTH = false;

    private static final String TAG_ACK = "ack";
    private static final String NAME_ACK = "Acknowledge";

    private static final String TAG_HOME = "home";
    private static final String NAME_HOME = "Home Registration";

    private static final String TAG_SINGLE = "single";
    private static final String NAME_SINGLE = "Single Address Only";

    private static final String TAG_DUP = "dup";
    private static final String NAME_DUP = "Duplicate Address Detection";

    private static final String TAG_SEQ = "seq";
    private static final String NAME_SEQ = "Sequence";

    private static final String TAG_LIFETIME = "lifetime";
    private static final String NAME_LIFETIME = "Lifetime";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

