/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv6_OPT_PADN extends Header implements IPv6Option, IPv6SubOption {

    public IPv6_OPT_PADN() {
	super();

        buffer = new byte[257];
        start = 0;
        end = 2;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_LENGTH, NAME_LENGTH);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = 1;
	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
	    FLAG_LENGTH = false;
            setLength((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_LENGTH = true;
        }
    }

    /* IPv6_OPT_PADN methods */

    public void setLength(byte length) {
	if(!FLAG_LENGTH) {
	    buffer[start+1] = length;
	    end = start + 2 + length;
	}
    }

    /* private IPv6_OPT_PADN methods */

    /* name */
    private static final String TAG_NAME = "IPv6_OPT_PADN";
    private static final String NAME_NAME = "IPv6 Option Pad N (PADN)";

    /* tags */
    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Length (excluding type and length fields)";
    private boolean FLAG_LENGTH = false;

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

