/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv6_OPT_ROUTING_TYPE0 extends Header implements IPv6Option {

    private int addr_ptr;

    public IPv6_OPT_ROUTING_TYPE0() {
	super();

        buffer = new byte[2040];
        start = 0;
        end = 4;
	addr_ptr = 4;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_ADDR, NAME_ADDR);
        return(h);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_ADDR.equals(tag)) {
            PacketUtils.debugln(5, TAG_ADDR + ": " + new String(ch, offset, length));
            StringTokenizer stok = new StringTokenizer(new String(ch, offset, length), ",");
            while(stok.hasMoreTokens()) {
                try {
                    setAddress(Inet6Address.getByName(stok.nextToken().trim()).getAddress());
                }
                catch(UnknownHostException e) {
                    PacketUtils.errdebugln(e.toString());
                    throw new SAXException(e);
                }
            }
        }
    }

    /* IPv6_OPT_ROUTING_TYPE0 methods */

    public void setLength(byte length) {
	/* This option does not take any suboptions, thus this method does nothing */
    }

    public void setAddress(byte[] addr) {
        int i;
	
        if(addr_ptr+16 >= buffer.length)
            return;

        for(i=0; i<16 && addr_ptr+i<buffer.length; i++)
            buffer[addr_ptr+i] = addr[i];

        addr_ptr += i;
	end = addr_ptr;
    }

    /* private IPv6_OPT_ROUTING_TYPE0 methods */

    /* name */
    private static final String TAG_NAME = "IPv6_OPT_ROUTING_TYPE0";
    private static final String NAME_NAME = "IPv6 Routing Option Type 0";

    /* tags */
    private static final String TAG_ADDR = "address";
    private static final String NAME_ADDR = "Addresses";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

