/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv6_OPT_SUB_BINDING_AUTH extends Header implements IPv6SubOption {

    public IPv6_OPT_SUB_BINDING_AUTH() {
	super();

        buffer = new byte[257];
        start = 0;
        end = 6;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_LENGTH, NAME_LENGTH);
	h.put(TAG_SPI, NAME_SPI);
	h.put(TAG_AUTHDATA, NAME_AUTHDATA);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = (byte) 4;
	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
            FLAG_LENGTH = false;
            setLength((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
            FLAG_LENGTH = true;
        }
	else if(TAG_SPI.equals(tag)) {
            PacketUtils.debugln(5, TAG_SPI + ": " + new String(ch, offset, length));
            setSPI(PacketUtils.parseInt(new String(ch, offset, length)));
        }
	else if(TAG_AUTHDATA.equals(tag)) {
            PacketUtils.debugln(5, TAG_AUTHDATA + ": " + new String(ch, offset, length));
	    setAuthData((new String(ch, offset, length)).getBytes());
	}
    }

    /* IPv6_OPT_SUB_BINDING_AUTH methods */

    public void setLength(byte length) {
	buffer[start+1] = length;
    }

    public void setSPI(int spi) {
	buffer[start+2] = (byte) ((spi & 0xff000000) >>> 24);
	buffer[start+3] = (byte) ((spi & 0x00ff0000) >>> 16);
	buffer[start+4] = (byte) ((spi & 0x0000ff00) >>> 8);
	buffer[start+5] = (byte) ((spi & 0x000000ff));
    }

    public void setAuthData(byte[] data) {
	if(start+6+data.length > buffer.length)
	    end = buffer.length;
	else
	    end = start+6+data.length;

	System.arraycopy(data, 0, buffer, start, end-start);
    }

    /* private IPv6_OPT_SUB_BINDING_AUTH methods */

    /* name */
    private static final String TAG_NAME = "IPv6_OPT_SUB_BINDING_AUTH";
    private static final String NAME_NAME = "IPv6 Binding Update Sub Option Authentication Data";

    /* tags */
    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Length";
    private boolean FLAG_LENGTH = false;

    private static final String TAG_SPI = "spi";
    private static final String NAME_SPI = "Security Parameter Index";

    private static final String TAG_AUTHDATA = "auth";
    private static final String NAME_AUTHDATA = "Authentication Data";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

