/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class IPv6_ROUTING extends Header implements NextHeader, IPv6Ext {

    /* private variables */
    private byte[] options;
    private int options_offset;
    private int options_length;

    public IPv6_ROUTING() {
	super();

        buffer = new byte[4];
        start = 0;
        end = 4;

	options = null;
        options_offset = 0;
        options_length = 0;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_NEXTHEADER, NAME_NEXTHEADER);
	h.put(TAG_LENGTH, NAME_LENGTH);
	h.put(TAG_TYPE, NAME_TYPE);
	h.put(TAG_SEG, NAME_SEG);
        return(h);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_NEXTHEADER.equals(tag)) {
            PacketUtils.debugln(5, TAG_NEXTHEADER + ": " + new String(ch, offset, length));
	    FLAG_NEXTHEADER = false;
            setNextHeader((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_NEXTHEADER = true;
        }
	else if(TAG_LENGTH.equals(tag)) {
            PacketUtils.debugln(5, TAG_LENGTH + ": " + new String(ch, offset, length));
	    FLAG_LENGTH = false;
            setLength((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_LENGTH = true;
        }
	else if(TAG_TYPE.equals(tag)) {
            PacketUtils.debugln(5, TAG_TYPE + ": " + new String(ch, offset, length));
            setType((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
	else if(TAG_SEG.equals(tag)) {
            PacketUtils.debugln(5, TAG_SEG + ": " + new String(ch, offset, length));
            setSegmentsLeft((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
        }
    }

    /* IPv6_HOP methods */

    public void setNextHeader(byte headerType) {
	if(!FLAG_NEXTHEADER)
	    buffer[start] = headerType;
    }

    public void setLength(byte length) {
	if(!FLAG_LENGTH)
	    buffer[start+1] = length;
    }

    public void setType(byte type) {
	buffer[start+2] = type;
    }

    public void setSegmentsLeft(byte count) {
	buffer[start+3] = count;
    }

    public void setOptions(byte[] data, int offset, int length) {
        options = data;
        options_offset = offset;
        options_length = length;
    }

    /* static public methods */

    public static byte getNextHeader(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length <= 2 || data.length < offset + length)
            return(-1);
        return(data[offset]);
    }

    public static int getLength(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length <= 2 || data.length < offset + length)
            return(-1);
        return(data[offset+1]);
    }

    /* private IPv6_ROUTING methods */

    /* name */
    private static final String TAG_NAME = "IPv6_ROUTING";
    private static final String NAME_NAME = "IPv6 Routing Option Header";

    /* tags */
    private static final String TAG_NEXTHEADER = "nextheader";
    private static final String NAME_NEXTHEADER = "Next Header";
    private boolean FLAG_NEXTHEADER = false;

    private static final String TAG_LENGTH = "length";
    private static final String NAME_LENGTH = "Length in 8-octet words";
    private boolean FLAG_LENGTH = false;

    private static final String TAG_TYPE = "type";
    private static final String NAME_TYPE = "Routing Type";

    private static final String TAG_SEG = "segmentsleft";
    private static final String NAME_SEG = "Segments Left";

    /* protocol id */
    public static final byte PROTOCOL_ID = (byte) 43;
}

