/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class MIPv4_RegRep extends Header {

    // MIPv4_RegRep specific stuff
    private byte[] pseudoHeader;
    private byte[] payload;
    private int payload_offset;
    private int payload_length;

    public MIPv4_RegRep() {
	super();
	
	/* create buffer */
        buffer = new byte[20];
        start = 0;
        end = 20;
    }

    public MIPv4_RegRep(byte[] buffer, int offset) {
	super();
	
	/* create buffer */
        this.buffer = buffer;
        start = offset;
        end = start + 20;
    }


    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }
    
    public byte getProtocolID() {
	return(PROTOCOL_ID);
    }

    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
        h.put(TAG_CODE, NAME_CODE);
        h.put(TAG_LIFETIME, NAME_LIFETIME);
        h.put(TAG_HOMEADDR, NAME_HOMEADDR);
        h.put(TAG_HOMEAGENT, NAME_HOMEAGENT);
        h.put(TAG_ID, NAME_ID);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {    
	buffer[start] = (byte) 0x03;
	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
        if(TAG_CODE.equals(tag)) {
            PacketUtils.debugln(5, TAG_CODE + ": " + new String(ch, offset, length));
	    setCode(PacketUtils.parseByte(new String(ch, offset, length)));
        }
	else if(TAG_LIFETIME.equals(tag)) {
            PacketUtils.debugln(5, TAG_LIFETIME + ": " + new String(ch, offset, length));
            setLifetime(PacketUtils.parseInt(new String(ch, offset, length)));
        }
        else if(TAG_HOMEADDR.equals(tag)) {
            PacketUtils.debugln(5, TAG_HOMEADDR + ": " + new String(ch, offset, length));
            try {
                setHomeAddress(Inet4Address.getByName(new String(ch, offset, length)).getAddress());
            }
            catch(UnknownHostException e) {
                PacketUtils.errdebugln(e.toString());
                throw new SAXException(e);
            }
        }
        else if(TAG_HOMEAGENT.equals(tag)) {
            PacketUtils.debugln(5, TAG_HOMEAGENT + ": " + new String(ch, offset, length));
            try {
                setHomeAgent(Inet4Address.getByName(new String(ch, offset, length)).getAddress());
            }
            catch(UnknownHostException e) {
                PacketUtils.errdebugln(e.toString());
                throw new SAXException(e);
            }
        }
        else if(TAG_ID.equals(tag)) {
            PacketUtils.debugln(5, TAG_ID + ": " + new String(ch, offset, length));
            setID(PacketUtils.parseLong(new String(ch, offset, length)));
        }
    }

    /* MIPv4_RegRep methods */

    public void setCode(byte code) {
	buffer[start+1] = code;
    }

    public void setLifetime(int lifetime) {
	buffer[start+2] = (byte) ((lifetime & 0xff00) >>> 8);
	buffer[start+3] = (byte) ((lifetime & 0x00ff));
    }

    public void setHomeAddress(Inet4Address address) {
        if(address != null)
            setHomeAddress(address.getAddress());
    }

    public void setHomeAddress(byte[] address) {
        if(address.length < 4)
            return;

        for(int i=0; i<4; i++)
            buffer[start+4+i] = address[i];
    }

    public void setHomeAgent(Inet4Address address) {
        if(address != null)
            setHomeAgent(address.getAddress());
    }

    public void setHomeAgent(byte[] address) {
        if(address.length < 4)
            return;

        for(int i=0; i<4; i++)
            buffer[start+8+i] = address[i];
    }

    public void setID(long id) {
	buffer[start+12] = (byte) ((id & 0xff00000000000000L) >>> 56);
	buffer[start+13] = (byte) ((id & 0x00ff000000000000L) >>> 48);
	buffer[start+14] = (byte) ((id & 0x0000ff0000000000L) >>> 40);
	buffer[start+15] = (byte) ((id & 0x000000ff00000000L) >>> 32);
	buffer[start+16] = (byte) ((id & 0x00000000ff000000L) >>> 24);
	buffer[start+17] = (byte) ((id & 0x0000000000ff0000L) >>> 16);
	buffer[start+18] = (byte) ((id & 0x000000000000ff00L) >>> 8);
	buffer[start+19] = (byte) ((id & 0x00000000000000ffL));
    }

    public byte getMIPType () {
	return getMIPType (buffer, start, end-start);
    }
 
    public byte getCode () {
	return getCode (buffer, start, end-start);
    }

    public int getLifetime () {
	return getLifetime (buffer, start, end-start);
    }

    public Inet4Address getHomeAddress () {
	return getHomeAddress (buffer, start, end-start);
    }

    public Inet4Address getHomeAgent () {
	return getHomeAgent (buffer, start, end-start);
    }

    public long getID () {
	return getID (buffer, start, end-start);
    }

    /* static public methods */

    public static boolean identify(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20 || data[offset] != 3)
            return(false);
        return(true);
    }

    public static byte getMIPType(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 20)
            return(-1);
        return(data[offset]);
    }

    public static byte getCode(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 20)
            return(-1);
        return(data[offset+1]);
    }

    public static int getLifetime(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 20)
            return(-1);
        return(((data[offset+2] & 0xff) << 8) | (data[offset+3] & 0xff));
    }

    public static Inet4Address getHomeAddress(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(null);
        byte[] addr = new byte[4];
        System.arraycopy(data, offset+4, addr, 0, 4);
        try {
            return((Inet4Address) Inet4Address.getByAddress(addr));
        }
        catch(UnknownHostException exp) {
        }
        return(null);
    }

    public static Inet4Address getHomeAgent(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(null);
        byte[] addr = new byte[4];
        System.arraycopy(data, offset+8, addr, 0, 4);
        try {
            return((Inet4Address) Inet4Address.getByAddress(addr));
        }
        catch(UnknownHostException exp) {
        }
        return(null);
    }

    public static long getID(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(-1L);
	return(((data[offset+12] & 0xffL) << 56) |
	       ((data[offset+13] & 0xffL) << 48) |
	       ((data[offset+14] & 0xffL) << 40) |
	       ((data[offset+15] & 0xffL) << 32) |
	       ((data[offset+16] & 0xffL) << 24) |
	       ((data[offset+17] & 0xffL) << 16) |
	       ((data[offset+18] & 0xffL) << 8) |
	       ((data[offset+19] & 0xffL))
	       );
    }

    /* private MIPv4_RegRep methods */

    /* name */
    private static final String TAG_NAME = "MIPv4_RegRep";
    private static final String NAME_NAME = "Mobile IPv4 Registration Reply";

    /* tags */
    private static final String TAG_TYPE = "type";
    private static final String NAME_TYPE = "Type";

    private static final String TAG_CODE = "code";
    private static final String NAME_CODE = "Code";

    private static final String TAG_LIFETIME = "lifetime";
    private static final String NAME_LIFETIME = "Lifetime";

    private static final String TAG_HOMEADDR = "homeaddr";
    private static final String NAME_HOMEADDR = "Home Address";

    private static final String TAG_HOMEAGENT = "homeagent";
    private static final String NAME_HOMEAGENT = "Home Agent";

    private static final String TAG_ID = "id";
    private static final String NAME_ID = "Identification";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}
