/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

public class MIPv4_RegReq extends Header {

    // MIPv4_RegReq specific stuff
    private byte[] pseudoHeader;
    private byte[] payload;
    private int payload_offset;
    private int payload_length;

    public MIPv4_RegReq() {
	super();
	
	/* create buffer */
        buffer = new byte[24];
        start = 0;
        end = 24;
    }

    public MIPv4_RegReq(byte[] buffer, int offset) {
	super();
	
	/* create buffer */
        this.buffer = buffer;
        start = offset;
        end = start + 24;
    }


    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }
    
    public byte getProtocolID() {
	return(PROTOCOL_ID);
    }
    
    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
        h.put(TAG_BIT_S, NAME_BIT_S);
        h.put(TAG_BIT_B, NAME_BIT_B);
        h.put(TAG_BIT_D, NAME_BIT_D);
        h.put(TAG_BIT_M, NAME_BIT_M);
        h.put(TAG_BIT_G, NAME_BIT_G);
        h.put(TAG_BIT_T, NAME_BIT_T);
        h.put(TAG_LIFETIME, NAME_LIFETIME);
        h.put(TAG_HOMEADDR, NAME_HOMEADDR);
        h.put(TAG_HOMEAGENT, NAME_HOMEAGENT);
        h.put(TAG_COA, NAME_COA);
        h.put(TAG_ID, NAME_ID);
        return(h);
    }
    
    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	buffer[start] = (byte) 0x01;
	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
	if(TAG_BIT_S.equals(tag)) {
            PacketUtils.debugln(5, TAG_BIT_S + ": " + new String(ch, offset, length));
	    setS(PacketUtils.parseBoolean(new String(ch, offset, length)));
        }
        else if(TAG_BIT_B.equals(tag)) {
            PacketUtils.debugln(5, TAG_BIT_B + ": " + new String(ch, offset, length));
	    setB(PacketUtils.parseBoolean(new String(ch, offset, length)));
        }
        else if(TAG_BIT_D.equals(tag)) {
            PacketUtils.debugln(5, TAG_BIT_D + ": " + new String(ch, offset, length));
	    setD(PacketUtils.parseBoolean(new String(ch, offset, length)));
        }
	else if(TAG_BIT_M.equals(tag)) {
            PacketUtils.debugln(5, TAG_BIT_M + ": " + new String(ch, offset, length));
	    setM(PacketUtils.parseBoolean(new String(ch, offset, length)));
        }
        else if(TAG_BIT_G.equals(tag)) {
            PacketUtils.debugln(5, TAG_BIT_G + ": " + new String(ch, offset, length));
	    setG(PacketUtils.parseBoolean(new String(ch, offset, length)));
        }
        else if(TAG_BIT_T.equals(tag)) {
            PacketUtils.debugln(5, TAG_BIT_T + ": " + new String(ch, offset, length));
	    setT(PacketUtils.parseBoolean(new String(ch, offset, length)));
        }
	else if(TAG_LIFETIME.equals(tag)) {
            PacketUtils.debugln(5, TAG_LIFETIME + ": " + new String(ch, offset, length));
            setLifetime(PacketUtils.parseInt(new String(ch, offset, length)));
        }
        else if(TAG_HOMEADDR.equals(tag)) {
            PacketUtils.debugln(5, TAG_HOMEADDR + ": " + new String(ch, offset, length));
            try {
                setHomeAddress(Inet4Address.getByName(new String(ch, offset, length)).getAddress());
            }
            catch(UnknownHostException e) {
                PacketUtils.errdebugln(e.toString());
                throw new SAXException(e);
            }
        }
        else if(TAG_HOMEAGENT.equals(tag)) {
            PacketUtils.debugln(5, TAG_HOMEAGENT + ": " + new String(ch, offset, length));
            try {
                setHomeAgent(Inet4Address.getByName(new String(ch, offset, length)).getAddress());
            }
            catch(UnknownHostException e) {
                PacketUtils.errdebugln(e.toString());
                throw new SAXException(e);
            }
        }
        else if(TAG_COA.equals(tag)) {
            PacketUtils.debugln(5, TAG_COA + ": " + new String(ch, offset, length));
            try {
                setCOA(Inet4Address.getByName(new String(ch, offset, length)).getAddress());
            }
            catch(UnknownHostException e) {
                PacketUtils.errdebugln(e.toString());
                throw new SAXException(e);
            }
        }
        else if(TAG_ID.equals(tag)) {
            PacketUtils.debugln(5, TAG_ID + ": " + new String(ch, offset, length));
            setID(PacketUtils.parseLong(new String(ch, offset, length)));
        }
    }

    /* MIPv4_RegReq methods */

    public void setS(boolean b) {
	if(b)
            buffer[start+1] = (byte) (buffer[start+1] | 0x80);
        else
            buffer[start+1] = (byte) (buffer[start+1] | 0x7f);
    }

    public void setB(boolean b) {
	if(b)
            buffer[start+1] = (byte) (buffer[start+1] | 0x40);
        else
            buffer[start+1] = (byte) (buffer[start+1] | 0xbf);
    }

    public void setD(boolean b) {
	if(b)
            buffer[start+1] = (byte) (buffer[start+1] | 0x20);
        else
            buffer[start+1] = (byte) (buffer[start+1] | 0xdf);
    }

    public void setM(boolean b) {
	if(b)
            buffer[start+1] = (byte) (buffer[start+1] | 0x10);
        else
            buffer[start+1] = (byte) (buffer[start+1] | 0xef);
    }

    public void setG(boolean b) {
	if(b)
            buffer[start+1] = (byte) (buffer[start+1] | 0x08);
        else
            buffer[start+1] = (byte) (buffer[start+1] | 0xf7);
    }

    public void setT(boolean b) {
	if(b)
            buffer[start+1] = (byte) (buffer[start+1] | 0x02);
        else
            buffer[start+1] = (byte) (buffer[start+1] | 0xfd);
    }

    public void setLifetime(int lifetime) {
	buffer[start+2] = (byte) ((lifetime & 0xff00) >>> 8);
	buffer[start+3] = (byte) ((lifetime & 0x00ff));
    }

    public void setHomeAddress(Inet4Address address) {
        if(address != null)
            setHomeAddress(address.getAddress());
    }

    public void setHomeAddress(byte[] address) {
        if(address.length < 4)
            return;

        for(int i=0; i<4; i++)
            buffer[start+4+i] = address[i];
    }

    public void setHomeAgent(Inet4Address address) {
        if(address != null)
            setHomeAgent(address.getAddress());
    }

    public void setHomeAgent(byte[] address) {
        if(address.length < 4)
            return;

        for(int i=0; i<4; i++)
            buffer[start+8+i] = address[i];
    }

    public void setCOA(Inet4Address address) {
        if(address != null)
            setCOA(address.getAddress());
    }

    public void setCOA(byte[] address) {
        if(address.length < 4)
            return;

        for(int i=0; i<4; i++)
            buffer[start+12+i] = address[i];
    }

    public void setID(long id) {
	buffer[start+16] = (byte) ((id & 0xff00000000000000L) >>> 56);
	buffer[start+17] = (byte) ((id & 0x00ff000000000000L) >>> 48);
	buffer[start+18] = (byte) ((id & 0x0000ff0000000000L) >>> 40);
	buffer[start+19] = (byte) ((id & 0x000000ff00000000L) >>> 32);
	buffer[start+20] = (byte) ((id & 0x00000000ff000000L) >>> 24);
	buffer[start+21] = (byte) ((id & 0x0000000000ff0000L) >>> 16);
	buffer[start+22] = (byte) ((id & 0x000000000000ff00L) >>> 8);
	buffer[start+23] = (byte) ((id & 0x00000000000000ffL));
    }

    public byte getMIPType() {
	return getMIPType (buffer, start, end-start);
    }

    public boolean getS() {
	return getS (buffer, start, end-start);
    }

    public boolean getB() {
	return getB (buffer, start, end-start);
    }

    public boolean getD() {
	return getD (buffer, start, end-start);
    }

    public boolean getM() {
	return getM (buffer, start, end-start);
    }

    public boolean getG() {
	return getG (buffer, start, end-start);
    }

    public boolean getT() {
	return getT (buffer, start, end-start);
    }

    public int getLifetime() {
	return getLifetime (buffer, start, end-start);
    }

    public Inet4Address getHomeAddress() {
	return getHomeAddress (buffer, start, end-start);
    }

    public Inet4Address getHomeAgent() {
	return getHomeAgent (buffer, start, end-start);
    }

    public Inet4Address getCOA() {
	return getCOA (buffer, start, end-start);
    }

    public long getID() {
	return getID (buffer, start, end-start);
    }

    /* static public methods */

    public static boolean identify(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 24 || data[offset] != 1)
            return(false);
        return(true);
    }

    public static byte getMIPType(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(-1);
        return(data[offset]);
    }

    public static boolean getS(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(false);
        return((data[offset+1] & 0x80) == 0x80);
    }

    public static boolean getB(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(false);
        return((data[offset+1] & 0x40) == 0x40);
    }

    public static boolean getD(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(false);
        return((data[offset+1] & 0x20) == 0x20);
    }

    public static boolean getM(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(false);
        return((data[offset+1] & 0x10) == 0x10);
    }

    public static boolean getG(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(false);
        return((data[offset+1] & 0x08) == 0x08);
    }

    public static boolean getT(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(false);
        return((data[offset+1] & 0x02) == 0x02);
    }

    public static int getLifetime(byte[] data, int offset, int length) {
	if(data == null || offset < 0 || length < 24)
            return(-1);
        return(((data[offset+2] & 0xff) << 8) | (data[offset+3] & 0xff));
    }

    public static Inet4Address getHomeAddress(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 24)
            return(null);
        byte[] addr = new byte[4];
        System.arraycopy(data, offset+4, addr, 0, 4);
        try {
            return((Inet4Address) Inet4Address.getByAddress(addr));
        }
        catch(UnknownHostException exp) {
        }
        return(null);
    }

    public static Inet4Address getHomeAgent(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 24)
            return(null);
        byte[] addr = new byte[4];
        System.arraycopy(data, offset+8, addr, 0, 4);
        try {
            return((Inet4Address) Inet4Address.getByAddress(addr));
        }
        catch(UnknownHostException exp) {
        }
        return(null);
    }

    public static Inet4Address getCOA(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 24)
            return(null);
        byte[] addr = new byte[4];
        System.arraycopy(data, offset+12, addr, 0, 4);
        try {
            return((Inet4Address) Inet4Address.getByAddress(addr));
        }
        catch(UnknownHostException exp) {
        }
        return(null);
    }

    public static long getID(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 24)
            return(-1L);
	return(((data[offset+16] & 0xffL) << 56) |
	       ((data[offset+17] & 0xffL) << 48) |
	       ((data[offset+18] & 0xffL) << 40) |
	       ((data[offset+19] & 0xffL) << 32) |
	       ((data[offset+20] & 0xffL) << 24) |
	       ((data[offset+21] & 0xffL) << 16) |
	       ((data[offset+22] & 0xffL) << 8) |
	       ((data[offset+23] & 0xffL))
	       );
    }

    /* private MIPv4_RegReq methods */

    /* name */
    private static final String TAG_NAME = "MIPv4_RegReq";
    private static final String NAME_NAME = "Mobile IPv4 Registration Request";

    /* tags */
    private static final String TAG_TYPE = "type";
    private static final String NAME_TYPE = "Type";

    private static final String TAG_BIT_S = "s";
    private static final String NAME_BIT_S = "Simultaneous Binding";

    private static final String TAG_BIT_B = "b";
    private static final String NAME_BIT_B = "Broadcast Datagrams";

    private static final String TAG_BIT_D = "d";
    private static final String NAME_BIT_D = "Decapsulation by MN";

    private static final String TAG_BIT_M = "m";
    private static final String NAME_BIT_M = "Minimal Encapsulation";

    private static final String TAG_BIT_G = "g";
    private static final String NAME_BIT_G = "GRE Encapsulation";

    private static final String TAG_BIT_T = "t";
    private static final String NAME_BIT_T = "Reverse Tunneling";

    private static final String TAG_LIFETIME = "lifetime";
    private static final String NAME_LIFETIME = "Lifetime";

    private static final String TAG_HOMEADDR = "homeaddr";
    private static final String NAME_HOMEADDR = "Home Address";

    private static final String TAG_HOMEAGENT = "homeagent";
    private static final String NAME_HOMEAGENT = "Home Agent";

    private static final String TAG_COA = "coa";
    private static final String NAME_COA = "Care-of Address";

    private static final String TAG_ID = "id";
    private static final String NAME_ID = "Identification";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}
