/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetengine.header;

import packetengine.*;
import java.util.*;
import java.net.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class TCP extends Header {

    private byte[] pseudoHeader;

    private byte[] options = null;
    private int options_offset = 0;
    private int options_length = 0;

    private byte[] payload;
    private int payload_offset;
    private int payload_length;

    public TCP() {
	super();

	/* create buffer */
        buffer = new byte[20];
        start = 0;
        end = 20;

	pseudoHeader = null;

	options = null;
	options_offset = 0;
	options_length = 0;

	payload = null;
	payload_offset = 0;
	payload_length = 0;
    }

    public TCP(byte[] buffer, int offset) {
	super();

	/* set buffer */
        this.buffer = buffer;
        start = offset;
        end = start + 20;

	pseudoHeader = null;

	options = null;
	options_offset = 0;
	options_length = 0;

	payload = null;
	payload_offset = 0;
	payload_length = 0;
    }

    /* Header methods */
    
    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public String getHeaderType() {
        return(TAG_NAME);
    }

	public byte getProtocolID() {
		return(PROTOCOL_ID);
	}


    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
        h.put(TAG_SPORT, NAME_SPORT);
        h.put(TAG_DPORT, NAME_DPORT);
        h.put(TAG_SEQNUM, NAME_SEQNUM);
        h.put(TAG_ACKNUM, NAME_ACKNUM);
        h.put(TAG_HL, NAME_HL);
        h.put(TAG_URG, NAME_URG);
        h.put(TAG_ACK, NAME_ACK);
        h.put(TAG_PSH, NAME_PSH);
        h.put(TAG_RST, NAME_RST);
        h.put(TAG_SYN, NAME_SYN);
        h.put(TAG_FIN, NAME_FIN);
        h.put(TAG_WSIZE, NAME_WSIZE);
        h.put(TAG_CSUM, NAME_CSUM);
        h.put(TAG_UPTR, NAME_UPTR);
        return(h);
    }

    public int build(byte[] buffer, int offset, int length, LinkedList headerList, int index) {
	Header h = null;

	if(buffer != null && offset >= 0 && offset+length <= buffer.length)
	    setPayload(buffer, offset, length);

	/* csum TCP */
	if(headerList != null && index < headerList.size()) {
	    for(int j=index-1; j>=0; j--) {
		h = (Header) headerList.get(j);
		if(h instanceof IPv4 || h instanceof IPv6)
		    break;
		h = null;
	    }
	    if(h != null && h instanceof IPv4)
		setPseudoHeader(PacketUtils.buildIPv4PseudoHeader((IPv4) h, PROTOCOL_ID, payload_length + getLength()));
	    else if(h != null && h instanceof IPv6) {
		int payload_length = getPayloadLength();
		setPseudoHeader(PacketUtils.buildIPv6PseudoHeader((IPv6) h, null, null, payload_length, PROTOCOL_ID)); 
	    }
	}

	if(!FLAG_SEQNUM)
            setSequence((new Random()).nextInt());
	if(!FLAG_ACKNUM)
            setAcknowledgement((new Random()).nextInt());
	if(!FLAG_HDRLEN)
	    setTCPHeaderLength((byte) ((int) (Math.ceil( (double) (getLength() + options_length) / (double) 4)) & 0xff));
	if(!FLAG_CSUM) {
	    if(pseudoHeader != null && payload != null) {
		setChecksum(0);
		setChecksum(PacketUtils.calcTCPChecksum(pseudoHeader, getBytes(), payload, payload_offset, payload_length));
		pseudoHeader = null;
		payload = null;
	    }
	}

	return(0);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
        if(TAG_SPORT.equals(tag)) {
            PacketUtils.debugln(5, TAG_SPORT + ": " + new String(ch, offset, length));
            setSourcePort(PacketUtils.parseInt(new String(ch, offset, length)));
        }
        else if(TAG_DPORT.equals(tag)) {
            PacketUtils.debugln(5, TAG_DPORT + ": " + new String(ch, offset, length));
            setDestinationPort(PacketUtils.parseInt(new String(ch, offset, length)));
        }
        else if(TAG_SEQNUM.equals(tag)) {
            PacketUtils.debugln(5, TAG_SEQNUM + ": " + new String(ch, offset, length));
	    FLAG_SEQNUM = false;
            setSequence(PacketUtils.parseInt(new String(ch, offset, length)));
	    FLAG_SEQNUM = true;
        }
        else if(TAG_ACKNUM.equals(tag)) {
            PacketUtils.debugln(5, TAG_ACKNUM + ": " + new String(ch, offset, length));
	    FLAG_ACKNUM = false;
            setAcknowledgement(PacketUtils.parseInt(new String(ch, offset, length)));
	    FLAG_ACKNUM = true;
        }
        else if(TAG_HL.equals(tag)) {
            PacketUtils.debugln(5, TAG_HL + ": " + new String(ch, offset, length));
	    FLAG_HDRLEN = false;
            setTCPHeaderLength((byte)(PacketUtils.parseInt(new String(ch, offset, length)) & 0xff));
	    FLAG_HDRLEN = true;
	}
        else if(TAG_URG.equals(tag)) {
            PacketUtils.debugln(5, TAG_URG + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setURG(true);
            else
                setURG(false);
        }
        else if(TAG_ACK.equals(tag)) {
            PacketUtils.debugln(5, TAG_ACK + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setACK(true);
            else
                setACK(false);
        }
        else if(TAG_PSH.equals(tag)) {
            PacketUtils.debugln(5, TAG_PSH + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setPSH(true);
            else
                setPSH(false);
        }
        else if(TAG_RST.equals(tag)) {
            PacketUtils.debugln(5, TAG_RST + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setRST(true);
            else
                setRST(false);
        }
        else if(TAG_SYN.equals(tag)) {
            PacketUtils.debugln(5, TAG_SYN + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setSYN(true);
            else
                setSYN(false);
        }
        else if(TAG_FIN.equals(tag)) {
            PacketUtils.debugln(5, TAG_FIN + ": " + new String(ch, offset, length));
	    if(PacketUtils.parseBoolean(new String(ch, offset, length)))
                setFIN(true);
            else
                setFIN(false);
        }
        else if(TAG_WSIZE.equals(tag)) {
            PacketUtils.debugln(5, TAG_WSIZE + ": " + new String(ch, offset, length));
            setWindowSize(PacketUtils.parseInt(new String(ch, offset, length)));
        }
        else if(TAG_CSUM.equals(tag)) {
            PacketUtils.debugln(5, TAG_CSUM + ": " + new String(ch, offset, length));
	    FLAG_CSUM = false;
            setChecksum(PacketUtils.parseInt(new String(ch, offset, length)));
	    FLAG_CSUM = true;
        }
        else if(TAG_UPTR.equals(tag)) {
            PacketUtils.debugln(5, TAG_UPTR + ": " + new String(ch, offset, length));
            setUrgentPointer(PacketUtils.parseInt(new String(ch, offset, length)));
        }
    }

    /* TCP methods */

    public void setSourcePort(int port) {
	PacketUtils.setU16(buffer, start, port);
    }

    public void setDestinationPort(int port) {
	PacketUtils.setU16(buffer, start+2, port);
    }

    public void setSequence(int seq) {
	PacketUtils.setU32(buffer, start+4, seq & 0xffffffffL);
    }

    public void setAcknowledgement(int ack) {
	PacketUtils.setU32(buffer, start+8, ack & 0xffffffffL);
    }

    public void setTCPHeaderLength(byte length) {
	if(!FLAG_HDRLEN) {
	    buffer[start+12] = (byte) (((length & 0x0f) << 4) |
				       (buffer[start+12] & 0x0f)); 
	}
    }

    public void setURG(boolean b) {
	if(b)
	    buffer[start+13] = PacketUtils.setMask(buffer[start+13], (byte) 0x20);
	else
	    buffer[start+13] = PacketUtils.clearMask(buffer[start+13], (byte) 0xdf);
    }

    public void setACK(boolean b) {
	if(b)
	    buffer[start+13] = PacketUtils.setMask(buffer[start+13], (byte) 0x10);
	else
	    buffer[start+13] = PacketUtils.clearMask(buffer[start+13], (byte) 0xef);
    }

    public void setPSH(boolean b) {
	if(b)
	    buffer[start+13] = PacketUtils.setMask(buffer[start+13], (byte) 0x08);
	else
	    buffer[start+13] = PacketUtils.clearMask(buffer[start+13], (byte) 0xf7);
    }

    public void setRST(boolean b) {
	if(b)
	    buffer[start+13] = PacketUtils.setMask(buffer[start+13], (byte) 0x04);
	else
	    buffer[start+13] = PacketUtils.clearMask(buffer[start+13], (byte) 0xfb);
    }

    public void setSYN(boolean b) {
	if(b)
	    buffer[start+13] = PacketUtils.setMask(buffer[start+13], (byte) 0x02);
	else
	    buffer[start+13] = PacketUtils.clearMask(buffer[start+13], (byte) 0xfd);
    }

    public void setFIN(boolean b) {
	if(b)
	    buffer[start+13] = PacketUtils.setMask(buffer[start+13], (byte) 0x01);
	else
	    buffer[start+13] = PacketUtils.clearMask(buffer[start+13], (byte) 0xfe);
    }

    public void setWindowSize(int size) {
	PacketUtils.setU16(buffer, start+14, size);
    }

    public void setChecksum(int csum) {
	if(!FLAG_CSUM)
	    PacketUtils.setU16(buffer, start+16, csum);
    }

    public void setUrgentPointer(int ptr) {
	PacketUtils.setU16(buffer, start+18, ptr);
    }

    public void setPseudoHeader(byte[] header) {
	pseudoHeader = header;
    }

    public void setPayload(byte[] data, int offset, int length) {
        payload = data;
	payload_offset = offset;
	payload_length = length;
    }

    public int getPayloadLength() {
	return(payload_length);
    }

    public int getSourcePort() {
	return getSourcePort(buffer, start, end-start);
    }

    public int getDestinationPort() {
	return getDestinationPort(buffer, start, end-start);
    }

    public long getSequence() {
	return getSequence(buffer, start, end-start);
    }

    public byte getTCPHeaderLength() {
	return getTCPHeaderLength(buffer, start, end-start);
    }

    public boolean getURG() {
	return getURG(buffer, start, end-start);
    }

    public boolean getACK() {
	return getACK(buffer, start, end-start);
    }

    public boolean getPSH() {
	return getPSH(buffer, start, end-start);
    }

    public boolean getRST() {
	return getRST(buffer, start, end-start);
    }

    public boolean getSYN() {
	return getSYN(buffer, start, end-start);
    }

    public boolean getFIN() {
	return getFIN(buffer, start, end-start);
    }

    /* static public methods */

    public static boolean identify(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(false);
        return(true);
    }

    public static int getSourcePort(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(-1);
	return(PacketUtils.getU16(data, offset));
    }

    public static int getDestinationPort(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(-1);
	return(PacketUtils.getU16(data, offset+2));
    }

    public static long getSequence(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(-1);
	return(PacketUtils.getU32(data, offset+4));
    }

    public static byte getTCPHeaderLength(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(-1);
	return((byte) ((data[offset+12] & 0xf0) >>> 4));
    }

    public static boolean getURG(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(false);
	return((data[offset+13] & 0x20) == 0x20);
    }

    public static boolean getACK(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(false);
	return((data[offset+13] & 0x10) == 0x10);
    }

    public static boolean getPSH(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(false);
	return((data[offset+13] & 0x08) == 0x08);
    }

    public static boolean getRST(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(false);
	return((data[offset+13] & 0x04) == 0x04);
    }

    public static boolean getSYN(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(false);
	return((data[offset+13] & 0x02) == 0x02);
    }

    public static boolean getFIN(byte[] data, int offset, int length) {
        if(data == null || offset < 0 || length < 20)
            return(false);
	return((data[offset+13] & 0x01) == 0x01);
    }

    /* private methods */

    /* name */
    private static final String TAG_NAME = "TCP";
    private static final String NAME_NAME = "Transmission Control Protocol";

    /* tags */
    private static final String TAG_SPORT = "srcport";
    private static final String NAME_SPORT = "Source Port";

    private static final String TAG_DPORT = "dstport";
    private static final String NAME_DPORT = "Destination Port";

    private static final String TAG_SEQNUM = "seqnum";
    private static final String NAME_SEQNUM = "Sequence Number";
    private boolean FLAG_SEQNUM = false;

    private static final String TAG_ACKNUM = "acknum";
    private static final String NAME_ACKNUM = "Acknowledgement Number";
    private boolean FLAG_ACKNUM = false;

    private static final String TAG_HL = "hlength";
    private static final String NAME_HL = "TCP Header Length";
    private boolean FLAG_HDRLEN = false;

    private static final String TAG_URG = "urg";
    private static final String NAME_URG = "Urgent Bit (URG)";

    private static final String TAG_ACK = "ack";
    private static final String NAME_ACK = "Acknowledgement Bit (ACK)";

    private static final String TAG_PSH = "psh";
    private static final String NAME_PSH = "Push Bit (PSH)";

    private static final String TAG_RST = "rst";
    private static final String NAME_RST = "Reset Bit (RST)";

    private static final String TAG_SYN = "syn";
    private static final String NAME_SYN = "Synchronize Bit (SYN)";

    private static final String TAG_FIN = "fin";
    private static final String NAME_FIN = "Finish Bit (FIN)";

    private static final String TAG_WSIZE = "winsize";
    private static final String NAME_WSIZE = "Window Size";

    private static final String TAG_CSUM = "csum";
    private static final String NAME_CSUM = "Checksum";
    private boolean FLAG_CSUM = false;

    private static final String TAG_UPTR = "urgptr";
    private static final String NAME_UPTR = "Urgent Pointer";

    /* protocol id */
    public static final byte PROTOCOL_ID = (byte) 6;
}
