/*
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netinet/ip.h>
#include <netinet/ip6.h>
#include <unistd.h>
#include <linux/unistd.h>
#include <string.h>
#include <errno.h>
#include "PacketSender.h"

static char *error_msg = NULL;

extern int errno;

JNIEXPORT jint JNICALL Java_packetsender_PacketSender_SendRawIPv4 (JNIEnv *env, jobject thisObj, jbyteArray packet, jint pktlen) {
  struct iphdr *ip;
  struct sockaddr_in dst_addr4;
  int i, s;

  jbyte *jArray = (*env)->GetByteArrayElements(env, packet, 0);

  ip = (struct iphdr *)jArray;

  memset(&dst_addr4, 0, sizeof(dst_addr4));
  dst_addr4.sin_family = AF_INET;
  dst_addr4.sin_addr.s_addr = ip->daddr;
  
  s = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
  if (s < 0) {
    error_msg = strerror(errno);
    perror("Error while opening socket type SOCK_RAW family AF_INET proto IPPROTO_RAW");
    return -1;
  }

  i=1;
  if(setsockopt(s, IPPROTO_IP, IP_HDRINCL, (char *) &i, sizeof(i)) < 0) {
    error_msg = strerror(errno);
    perror("Error while setting socket option IP_HDRINCL");
    return -2;
  }

  i = sendto(s, (void *)jArray, pktlen, 0x0, (struct sockaddr *) &dst_addr4, sizeof(dst_addr4));

  if ( i != pktlen) {
    error_msg = strerror(errno);
    perror("Error while sending through RAW socket");
  }

  if(s != 0)
    close(s);

  return i;
}

JNIEXPORT jint JNICALL Java_packetsender_PacketSender_SendRawIPv6 (JNIEnv *env, jobject thisObj, jbyteArray packet, jint pktlen) {
  struct ip6_hdr *ip6;
  struct sockaddr_in6 dst_addr6;
  int i, s;

  jbyte *jArray = (*env)->GetByteArrayElements(env, packet, 0);

  ip6 = (struct ip6_hdr *)jArray;

  memset(&dst_addr6, 0, sizeof(dst_addr6));
  dst_addr6.sin6_family = AF_INET6;
  memcpy((void *) &dst_addr6.sin6_addr, (void *) &ip6->ip6_dst, 16);
  
  s = socket(PF_INET6, SOCK_RAW, IPPROTO_RAW);
  if (s < 0) {
    error_msg = strerror(errno);
    perror("Error while opening socket type SOCK_RAW family AF_INET6 proto IPPROTO_RAW");
    return -1;
  }

  i=1;
  if(setsockopt(s, IPPROTO_IP, IP_HDRINCL, (char *) &i, sizeof(i)) < 0) {
    error_msg = strerror(errno);
    perror("Error while setting socket option IP_HDRINCL");
    return -2;
  }

  i = sendto(s, (void *)jArray, pktlen, 0x0, (struct sockaddr *) &dst_addr6, sizeof(dst_addr6));

  if ( i != pktlen) {
    error_msg = strerror(errno);
    perror("Error while sending through RAW socket");
  }

  if(s != 0)
    close(s);

  return i;
}

JNIEXPORT jstring JNICALL Java_packetsender_PacketSender_getErrorString (JNIEnv *env, jobject this) {
  if(error_msg != NULL)
    return((*env)->NewStringUTF(env, error_msg)); 

  return(NULL);
}
