/**
 *
 * Copyright (C) 2002-2003 VTT Information Technology. All rights reserved.
 *
 * This software is provided AS-IS with no warranty, either expressed or 
 * implied.
 *
 * This software is distributed under licence and may not be copied, modified 
 * or distributed except as expressly authorised under the terms of the licence
 * contained in the file LICENSE in this distribution.
 *
 * For more information about licensing, please refer to
 * http://opensource.erve.vtt.fi/ or contact VTT Information Technology.
 *
 */
package packetsender;

import packetengine.*;

public class PacketSender extends NetworkInterface {
  
  /**
   * Default constructor does nothing
   */
  public PacketSender() {

  }

  // NetworkInterface methods

  public static String getIFTag() {
    return(TAG_IF);
  }

  public boolean setParams(String params) {
    return(true);
  }

  public boolean ifup() {
    return(true);
  }

  public boolean ifdown() {
    return(true);
  }

  public boolean sendData(byte[] packet) {
    boolean retval = false;

    if((packet[0] & 0xf0) == 0x40)
      retval = sendIPv4Packet(packet);
    else if((packet[0] & 0xf0) == 0x60)
      retval = sendIPv6Packet(packet);
	
    if(!retval)
      PacketUtils.errdebugln("RawSocket could not send packet to network: " + getErrorString());

    return(retval);
  }

  // Java Native Interface methods

  public native int SendRawIPv4(byte[] packet, int pktlen);
  public native int SendRawIPv6(byte[] packet, int pktlen);
  public native String getErrorString();
    
  // Load the native part
  static
  {
    System.loadLibrary("RawSender");
  }

  // private PacketSender methods

  private boolean sendIPv4Packet(byte[] packet) {
    if(SendRawIPv4(packet, packet.length) != packet.length)
      return(false);

    return(true);
  }

  private boolean sendIPv6Packet(byte[] packet) {
    if(SendRawIPv6(packet, packet.length) != packet.length)
      return(false);
    
    return(true); 
  }

  // static variables

  private static final String TAG_IF = "RawSocket";
}
