package packetengine.header;

import packetengine.*;
import java.util.*;
import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;


public class Payload extends Header {

    public Payload() {
	super();

        buffer = new byte[65536];
        start = 0;
        end = 0;
    }

    /* Header methods */

    public String getHeaderType() {
	return(TAG_NAME);
    }

    public byte getProtocolID() {
	return(PROTOCOL_ID);
    }

    public static String getName() {
        return(NAME_NAME);
    }

    public static String getType() {
        return(TAG_NAME);
    }

    public static LinkedHashMap getTags() {
        LinkedHashMap h = new LinkedHashMap();
	h.put(TAG_DATA, NAME_DATA);
	h.put(TAG_FMTDATA, NAME_FMTDATA);
	h.put(TAG_HEXDATA, NAME_HEXDATA);
	h.put(TAG_FILEDATA, NAME_FILEDATA);
        return(h);
    }

    /* DefaultHandler methods */

    public void characters(char[] ch, int offset, int length) throws SAXException {
        if(TAG_DATA.equals(tag)) {
            PacketUtils.debugln(5, TAG_DATA + ": " + new String(ch, offset, length));
            setData((new String(ch, offset, length)).getBytes());
        }

        else if(TAG_FMTDATA.equals(tag)) {
            PacketUtils.debugln(5, TAG_FMTDATA + ": " + new String(ch, offset, length));
            setFormattedData(ch, offset, length);
        }

        else if(TAG_HEXDATA.equals(tag)) {
            PacketUtils.debugln(5, TAG_HEXDATA + ": " + new String(ch, offset, length));
            setHexData(ch, offset, length);
        }

        else if(TAG_FILEDATA.equals(tag)) {
            PacketUtils.debugln(5, TAG_FILEDATA + ": " + new String(ch, offset, length));
            setFileData(new String(ch, offset, length));
        }
    }

    /* Payload methods */

    public void setData(byte[] data, int data_start, int data_length) {
	int len = data_length;
	if(len > buffer.length - start)
	    len = buffer.length - start;

	System.arraycopy(data, data_start, buffer, start, len); 
	end = start + len;
    }

    public void setData(byte[] data) {
	int len = data.length;
	if(len > buffer.length - start)
	    len = buffer.length - start;
	
	System.arraycopy(data, 0, buffer, start, len);
	end = start + len;
    }

    public void setFormattedData(char[] char_data, int char_start, int char_length) {
        String str = new String(char_data, char_start, char_length);	
	String substr = str;

	for(int i=0; i<str.length(); i++) {
	    if(str.charAt(i) == '\\' && (i+1) < str.length()) {
		if(str.charAt(i+1) == 't') {
		    substr = str.substring(0, i) + "\t";
		    if((i+2) < str.length())
			substr = substr + str.substring(i+2);		    
		}
		else if(str.charAt(i+1) == 'r') {
		    substr = str.substring(0, i) + "\r";
		    if((i+2) < str.length())
			substr = substr + str.substring(i+2);		    
		}
		if(str.charAt(i+1) == 'n') {
		    substr = str.substring(0, i) + "\n";
		    if((i+2) < str.length())
			substr = substr + str.substring(i+2);		    
		}
		str = substr;
	    }
	}
	
	PacketUtils.debugln(5, TAG_FMTDATA + ": " + str);

	byte[] data = str.getBytes();

	int len = data.length;
	if(len > buffer.length - start)
	    len = buffer.length - start;

	System.arraycopy(data, 0, buffer, start, len); 
	end = start + len;
    }

    public void setHexData(char[] char_data, int char_start, int char_length) {
	int i=char_start;
	int j=start;

	while((i+1)<(char_start + char_length) && j < buffer.length) {

	    if(Character.isWhitespace(char_data[i])) {
		i++;
		continue;
	    }

	    else {
		buffer[j] = (byte) (0xff & 
			     Integer.parseInt(("" + char_data[i] + 
					      "" + char_data[i+1]).trim(), 
					      16));
		j++;
		i+=2;
	    }
	}

	if(i < (char_start + char_length)) {
		buffer[j] = (byte) (0xff & 
			     Integer.parseInt(("" + char_data[i]).trim(), 16));
		j++;
	}
	
	end = j;
    }

    public void setFileData(String filename) {
        int rlen = 0;
        int ptr = start+4;
        try {
            FileInputStream fish = new FileInputStream(filename);
            while(true) {
                rlen = fish.read(buffer, ptr, buffer.length - ptr);
                if(rlen < 0)
                    break;
                ptr += rlen;
            }

            end = ptr;
            PacketUtils.debugln(5, TAG_FILEDATA + ": payload file length = " + (end -
start - 4));
        }
        catch(Exception exp) {
            PacketUtils.errdebugln(TAG_FILEDATA + ": " + exp.toString());
        }
    }    

    /* private Payload methods */

    /* variables */

    /* name */
    private static final String TAG_NAME = "Payload";
    private static final String NAME_NAME = "Payload";

    /* tags */
    private static final String TAG_DATA = "data";
    private static final String NAME_DATA = "Payload Data";

    private static final String TAG_FMTDATA = "formatteddata";
    private static final String NAME_FMTDATA = "Formatted Data";

    private static final String TAG_HEXDATA = "hexdata";
    private static final String NAME_HEXDATA = "Hexadecimal Data";

    private static final String TAG_FILEDATA = "file";
    private static final String NAME_FILEDATA = "Payload Data Filename";

    /* protocol id */
    public static final byte PROTOCOL_ID = Header.UNDEFINED_PROTOCOL;
}

